<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://getwpfunnels.com/
 * @since      1.0.0
 *
 * @package    Wpfunnels_Pro_Lms
 * @subpackage Wpfunnels_Pro_Lms/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Wpfunnels_Pro_Lms
 * @subpackage Wpfunnels_Pro_Lms/admin
 * @author     WPFunnels <support@getwpfunnels.com>
 */

use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;
use WPFunnels\lms\helper\Wpfnl_lms_functions;

class Wpfunnels_Pro_Lms_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Wpfunnels_Pro_Lms_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Wpfunnels_Pro_Lms_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/wpfunnels-pro-lms-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Wpfunnels_Pro_Lms_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Wpfunnels_Pro_Lms_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/wpfunnels-pro-lms-admin.js', array( 'jquery' ), $this->version, false );

	}

	/**
	 * Enable lms settings
	 * 
	 */
	public function enable_lms_settings(){
		
		return true;
	}

	/**
	 * Create lms database
	 * @since 2.4.4
	 */

	public function create_lms_order_table()
	{
		
		$is_lms_order_table = get_option('wpfnl_lms_order_tables_created');
		if(!$is_lms_order_table){
			update_option('wpfnl_lms_order_tables_created','yes');
			Wpfnl_lms_learndash_functions::create_lms_order_table();
		}
	}


	/**
	 * Include lms block type
	 */
	public function include_lms_block_type( $blocks ){
		
		if( !in_array('LmsCheckout',$blocks) ){
			array_push($blocks,'LmsCheckout');
		}
		if( !in_array('LmsOrderDetails',$blocks) ){
			array_push($blocks,'LmsOrderDetails');
		}
		
		return $blocks;
	}


	/**
	 * get template data
	 */
	public function get_template_data( $data ){
		if( 'yes' === get_option('_wpfnl_updated_template_data_') && 'active' == get_option( 'wpfunnels_pro_lms_license_status' ) ){
			return $data;
		}else{
			update_option( '_wpfnl_updated_template_data_','yes' );
			return false;
		}
	}


	/**
     * register addon for licensing
     *
     * @param $addons
     * @return mixed
     *
     * @since 1.0.3
     */
    public function register_addon_for_licensing( $addons ) {

		if ( ! function_exists( 'get_plugins' ) || ! function_exists( 'is_plugin_active' ) ) {
            require_once ABSPATH . 'wp-admin/includes/plugin.php';
        }

        $wp_function =  new \WPFunnelsPro\Wpfnl_Pro_functions();

		$addons['lms'] = array(
                'key'           => 'lms',
                'name'          => __( 'WPFunnels Pro - LMS Funnel', 'wpfunnels-pro-lms' ),
                'description'   => __( 'Get advanced sales funnel features such as Upsell, Downsell for your Online Courses.', 'wpfunnels-pro-lms' ),
                'icon'          => WPFNL_PRO_LMS_DIR . '/admin/partials/icons/lms-icon.php',
                'slug'          => 'wpfunnels-pro-lms/wpfunnels-pro-lms.php',
                'version'       => WPFNL_PRO_LMS_VERSION,
                'plugin_status' => $this->check_plugin_installed('wpfunnels-pro-lms/wpfunnels-pro-lms.php') ? 'installed' : 'not-installed',
                'license_key'   => get_option( 'wpfunnels_pro_lms_license_key'),
                'license_data'  => get_option( 'wpfunnels_pro_lms_license_data' ),
                'license_status'=> get_option( 'wpfunnels_pro_lms_license_status' ),
                'product_id'    =>  WPFNL_PRO_LMS_PRODUCT_ID,
                'dependency'    => 'pro',
                'btn_txt'       => $this->get_button_text('wpfunnels-pro-lms/wpfunnels-pro-lms.php'),
                'btn_link'      => $this->check_plugin_installed('wpfunnels-pro-lms/wpfunnels-pro-lms.php') ? admin_url('plugins.php') : 'https://getwpfunnels.com/pricing/',
		);
      

        return $addons;
    }


	/**
     * check if plugin is installed
     *
     * @param $plugin_slug
     * @return bool
     */
    public function check_plugin_installed( $plugin_slug ) {
        if ( ! function_exists( 'get_plugins' ) || ! function_exists( 'is_plugin_active' ) ) {
            require_once ABSPATH . 'wp-admin/includes/plugin.php';
        }
        $installed_plugins = get_plugins();
        return array_key_exists( $plugin_slug, $installed_plugins ) || in_array( $plugin_slug, $installed_plugins, true );
    }


	/**
     * get button text
     *
     * @param $slug
     * @return string|void
     *
     * @since 1.3.1
     */
    public function get_button_text( $slug ) {
        if ( !$this->check_plugin_installed( $slug ) ) {
            return __('Get the add-on', 'wpfnl-pro');
        }

        if ( is_plugin_active( $slug ) ) {
            return __('Activated', 'wpfnl-pro');
        }
        return __('Please enable', 'wpfnl-pro');
    }


	/**
	 * Modify template type
	 * 
	 * @return Array
	 * @since 1.0.0
	 * 
	 */
	public function modify_template_type( $types ){
		
		if( Wpfnl_lms_learndash_functions::is_learndash_active() && Wpfnl_lms_functions::is_pro_lms_license_activated() ){
			if( is_array($types) ){
				$new_type = [
					'slug'   => 'lms',
					'label'  => 'LMS Templates',
				];
				array_push($types,$new_type);
			}
		}
		return $types;
	}


	/**
	 * Delete transient for check update
	 * 
	 * @since 1.0.7
	 */
	public function check_update_transient(){
		delete_site_transient( 'update_plugins' );
		$this->clear_lms_update_check_transient();
	}


	/**
     * clear lms update check transient data
     *
     * @since 1.0.7
     */
    private function clear_lms_update_check_transient() {
        global $wpdb;
        $sql   = "DELETE a, b FROM $wpdb->options a, $wpdb->options b
		WHERE a.option_name LIKE %s
		AND a.option_name NOT LIKE %s
		AND b.option_name = CONCAT( '_site_transient_timeout_', SUBSTRING( a.option_name, 17 ) )";
        $wpdb->query( $wpdb->prepare( $sql, $wpdb->esc_like( '_site_transient_wpfunnelspro-lms-check_for_plugin_update' ) . '%', $wpdb->esc_like( '_site_transient_timeout_' ) . '%' ) );

    }


	



}
