<?php

namespace WPFunnels\lms\helper;
use WPFunnels\Wpfnl_functions;
class Wpfnl_lms_learndash_functions
{


    /**
     * Check leandash plugin is active or not
     * 
     * @return Bool
     * @since 1.0.0
     */
    public static function is_learndash_active(){
        if (in_array('sfwd-lms/sfwd_lms.php', apply_filters('active_plugins', get_option('active_plugins')))) {
			return true;
		}
		return false;
    }


    /**
	 * Get learndash settings by funnel ID from post meta
	 *
	 * @param $funnel_id
	 */
	public static function get_learndash_settings( $funnel_id = null ){

		if( $funnel_id ){
			$type = get_post_meta( $funnel_id , '_wpfnl_funnel_type' , true );
			if( 'lms' === $type ){
				return 'yes';
			}
		}
		return false;
	}


	/**
	 * get course by id
	 *
	 * @param $id
	 */
	public static function get_course_by_id( $id = null ){

		if( $id ){
			$args = array(
				'post_type' => 'sfwd-courses',
				'post_status' => 'publish',
				'post__in' =>  [$id],
			);
			$courses = get_posts($args);

			if( $courses ){
				return $courses[0];
			}
		}
		return false;
	}


	/**
	 * Search cousrse
	 *
	 * @param $term
	 */
	public static function search_course( $term = null ){

		if( $term ){
			$args = array(
				'post_type'   => 'sfwd-courses',
				's' 		  =>  $term,
				'post_status' => 'publish',
				'orderby'     => 'ID',
				'order'       => 'ASC'
			);
			$courses = get_posts($args);
			if( $courses){
				return $courses;
			}

		}
		return [];

	}


	/**
	 * @desc Get course details by Step ID
	 * @param $step_id
	 * @return array
	 */
	public static function get_course_details( $step_id )
	{
		$course_details = [];
		if( $step_id ) {
			$step_type = get_post_meta( $step_id, '_step_type', true );
			$product   = get_post_meta( $step_id, '_wpfnl_' . $step_type . '_products', true );

			if( $product && !empty( $product ) ) {
				$course_id   = isset( $product[ 0 ][ 'id' ] ) ? $product[ 0 ][ 'id' ] : '';
				$course      = self::get_course_by_id( $course_id );
				$image_id    = get_post_meta( $course_id, '_thumbnail_id', true );
				$image       = wp_get_attachment_image_src( $image_id, 'single-post-thumbnail' );
				$course_meta = learndash_get_course_meta_setting( $course_id );
				$discount_price = self::get_discounted_course_price( $step_id );
				$course_details = [
					'id'                 => $course_id,
					'title'              => isset( $course->post_title ) ? $course->post_title : '',
					'no_of_lesson'       => count( learndash_get_lesson_list( $course_id ) ),
					'type'               => isset( $course_meta[ 'sfwd-courses_course_price_type' ] ) ? $course_meta[ 'sfwd-courses_course_price_type' ] : '',
					'billing_cycle'      => isset( $course_meta[ 'sfwd-courses_course_price_billing_p3' ] ) ? $course_meta[ 'sfwd-courses_course_price_billing_p3' ] : '',
					'billing_cycle_unit' => isset( $course_meta[ 'sfwd-courses_course_price_billing_t3' ] ) ? $course_meta[ 'sfwd-courses_course_price_billing_t3' ] : '',
					'recurring_time'     => isset( $course_meta[ 'sfwd-courses_course_no_of_cycles' ] ) ? $course_meta[ 'sfwd-courses_course_no_of_cycles' ] : '',
					'price'              => isset( $course_meta[ 'sfwd-courses_course_price' ] ) ? $course_meta[ 'sfwd-courses_course_price' ] : '',
					'trial_price'        => isset( $course_meta[ 'sfwd-courses_course_trial_price' ] ) ? $course_meta[ 'sfwd-courses_course_trial_price' ] : '',
					'trial_period_unit'  => isset( $course_meta[ 'sfwd-courses_course_trial_duration_t1' ] ) ? $course_meta[ 'sfwd-courses_course_trial_duration_t1' ] : '',
					'trial_period'       => isset( $course_meta[ 'sfwd-courses_course_trial_duration_p1' ] ) ? $course_meta[ 'sfwd-courses_course_trial_duration_p1' ] : '',
					'is_expire'          => isset( $course_meta[ 'sfwd-courses_expire_access' ] ) ? $course_meta[ 'sfwd-courses_expire_access' ] : '',
					'expire_days'        => isset( $course_meta[ 'sfwd-courses_expire_access_days' ] ) ? $course_meta[ 'sfwd-courses_expire_access_days' ] : '',
					'image_id'           => $image_id ? $image_id : '',
					'image'              => isset( $image[ 0 ] ) ? $image[ 0 ] : '',
					'description'        => isset( $course_meta[ 'sfwd-courses_course_materials' ] ) ? $course_meta[ 'sfwd-courses_course_materials' ] : '',
					'currency'           => function_exists( 'learndash_get_currency_symbol' ) ? self::get_currency_symbol_from_country_code( learndash_get_currency_symbol() ) : self::get_currency_symbol_from_country_code( learndash_30_get_currency_symbol() ),
					'discount_price'     => $discount_price
				];
			}
		}
		return $course_details;
	}


	/**
	 * Get discounted price
	 * 
	 * @param String
	 * @return String
	 * @since 1.0.4
	 */
	public static function get_discounted_course_price( $step_id ){
		$price = '';
		
		if( $step_id ){
			$step_type = get_post_meta( $step_id, '_step_type', true);
			if( 'checkout' == $step_type ){
				$discount = get_post_meta( $step_id, '_wpfnl_checkout_discount_main_product', true );
				if( is_array($discount) ){
					if( isset($discount['discountOptions']) && isset($discount['mutedDiscountValue']) && 'original' != $discount['discountOptions'] && $discount['mutedDiscountValue'] ){
						if( isset($discount['discountedPrice']) ){
							$price = $discount['discountedPrice'];
						}
					}
				}
			}elseif( 'upsell' == $step_type || 'downsell' == $step_type ){
				$discount = get_post_meta( $step_id, '_wpfnl_'.$step_type.'_discount', true );
				$item = get_post_meta( $step_id, '_wpfnl_'.$step_type.'_products', true );
				$course_id 	= isset($item[0]['id']) ? $item[0]['id'] : '';
				$course 	= self::get_course_details_by_id( $course_id );

				if( is_array( $discount ) ){
					if( isset($discount['discountType']) && isset($discount['discountValue']) && 'original' != $discount['discountType'] && $discount['discountValue'] ){
						$price = Wpfnl_functions::calculate_discount_price( $discount['discountType'], $discount['discountValue'], $course['price'] );
					}
				}
			}
		}
		return $price;
	}


	/**
	 * Get course details by step Id
	 */
	public static function get_course_details_by_id( $course_id = null ){
		$course_details = [];
		if( $course_id ){

			$course = self::get_course_by_id($course_id);
			$image_id = get_post_meta($course_id,'_thumbnail_id', true);
			$image = wp_get_attachment_image_src($image_id, 'single-post-thumbnail');

			$course_meta = learndash_get_course_meta_setting($course_id);

			$course_details = [
				'id'						=> $course_id,
				'title' 					=> $course->post_title,
				'no_of_lesson' 				=> count(learndash_get_lesson_list($course_id)),
				'type' 						=> $course_meta['sfwd-courses_course_price_type'],
				'billing_cycle' 			=> $course_meta['sfwd-courses_course_price_billing_p3'],
				'billing_cycle_unit' 		=> $course_meta['sfwd-courses_course_price_billing_t3'],
				'recurring_time'			=> isset($course_meta['sfwd-courses_course_no_of_cycles']) ? $course_meta['sfwd-courses_course_no_of_cycles'] : '',
				'price' 					=> $course_meta['sfwd-courses_course_price'],
				'trial_price' 				=> $course_meta['sfwd-courses_course_trial_price'],
				'trial_period_unit' 		=> $course_meta['sfwd-courses_course_trial_duration_t1'],
				'trial_period' 				=> $course_meta['sfwd-courses_course_trial_duration_p1'],
				'is_expire' 				=> $course_meta['sfwd-courses_expire_access'],
				'expire_days' 				=> $course_meta['sfwd-courses_expire_access_days'],
				'image_id' 					=> $image_id ? $image_id : '',
				'image' 					=> isset($image[0]) ? $image[0] : '',
				'description' 				=> $course_meta['sfwd-courses_course_materials'],
				'currency' 				    => function_exists( 'learndash_get_currency_symbol' ) ? self::get_currency_symbol_from_country_code( learndash_get_currency_symbol() ) : self::get_currency_symbol_from_country_code( learndash_30_get_currency_symbol() ),
			];

		}
		
		return $course_details;
	}



	/**
	 * Get order details of current user for showing thank you page by user id
	 */
	public static function get_lms_order_details( $user_id ,$funnel_id ){
		global $wpdb;
		$table_name = $wpdb->prefix.'wpfnl_lms_order';
		$results = $wpdb->get_results( "SELECT * FROM ".$table_name." WHERE funnel_id = ".$funnel_id." AND user_id = ".$user_id." AND meta = 'new_order'");
		$response = [];
		$default_course_details = [
			'id'						=> 0,
			'title' 					=> 'Course Title',
			'no_of_lesson' 				=> 2,
			'type' 						=> 'paynow',
			'billing_cycle' 			=> '',
			'billing_cycle_unit' 		=> '',
			'recurring_time'			=> '',
			'price' 					=> '120',
			'discount_price' 					=> '120',
			'trial_price' 				=> '',
			'trial_period_unit' 		=> '',
			'trial_period' 				=> '',
			'is_expire' 				=> '',
			'expire_days' 				=> '',
			'image_id' 					=> '',
			'image' 					=> '',
			'description' 				=> 'Course description',
			'currency' 				    => '$',
		];
        $data        = \WPFunnels\Wpfnl_functions::get_sanitized_get_post();
		if( !empty($results) ){
			foreach( $results as $key=>$result ){
				$step_type = get_post_meta( $result->step_id , '_step_type', true );
				$course_details = unserialize($result->order_data); //phpcs:ignore
				$overview = [
					'step_type' 	 	 => $step_type,
					'payment_method' 	 => $result->payment_method,
					'amount' 	 		 => $result->amount,
					'course_details'     => $course_details,
				];

				array_push($response, $overview);
			}
		}elseif (!isset($data[ 'get' ][ 'wpfnl_ld_payment' ])){
			$overview = [
				'step_type' 	 => 'checkout',
				'payment_method' => 'Paypal',
				'amount' 	 	 => isset($results->amount) ? $results->amount : '',
				'course_details' => $default_course_details,
			];
			array_push($response, $overview);
		}
        return $response;

	}

	/**
	 * Get Next step Url
	 * @param $funnel_id
	 * @param $step_id
	 * @return false|string|\WP_Error
	 */

	public static function get_next_step_url ($funnel_id, $step_id){
		$next_step       = Wpfnl_functions::get_next_step( $funnel_id, $step_id );
		$redirect_url = '';
		if( isset( $next_step[ 'step_id' ] ) ) {
			if( $next_step[ 'step_type' ] == 'conditional' ) {
				$next_step = Wpfnl_functions::get_next_step( $funnel_id, $next_step[ 'step_id' ] );
			}
			$redirect_url = get_permalink( $next_step[ 'step_id' ] );
		}
		return $redirect_url;
	}



	/**
	 * get currency symbol by country code
	 */
	public static function get_currency_symbol_from_country_code( $country_code ){

		$currencySymbols = [
			'AED' => 'د.إ',
			'AFN' => 'Af',
			'ALL' => 'Lek',
			'AMD' => 'դ',
			'ANG' => 'ƒ',
			'AOA' => 'Kz',
			'ARS' => '$',
			'AUD' => '$',
			'AWG' => 'ƒ',
			'AZN' => '₼',
			'BAM' => 'KM',
			'BBD' => '$',
			'BDT' => '৳',
			'BGN' => 'лв',
			'BHD' => '.د.ب',
			'BIF' => 'FBu',
			'BMD' => '$',
			'BND' => '$',
			'BOB' => '$b',
			'BRL' => 'R$',
			'BSD' => '$',
			'BTN' => 'Nu.',
			'BWP' => 'P',
			'BYR' => 'p.',
			'BZD' => 'BZ$',
			'CAD' => '$',
			'CDF' => 'FC',
			'CHF' => 'CHF',
			'CLF' => 'UF',
			'CLP' => '$',
			'CNY' => '¥',
			'COP' => '$',
			'CRC' => '₡',
			'CUP' => '⃌',
			'CVE' => '$',
			'CZK' => 'Kč',
			'DJF' => 'Fdj',
			'DKK' => 'kr',
			'DOP' => 'RD$',
			'DZD' => 'دج',
			'EGP' => 'E£',
			'ETB' => 'Br',
			'EUR' => '€',
			'FJD' => '$',
			'FKP' => '£',
			'GBP' => '£',
			'GEL' => 'ლ',
			'GHS' => '¢',
			'GIP' => '£',
			'GMD' => 'D',
			'GNF' => 'FG',
			'GTQ' => 'Q',
			'GYD' => '$',
			'HKD' => '$',
			'HNL' => 'L',
			'HRK' => 'kn',
			'HTG' => 'G',
			'HUF' => 'Ft',
			'IDR' => 'Rp',
			'ILS' => '₪',
			'INR' => '₹',
			'IQD' => 'ع.د',
			'IRR' => '﷼',
			'ISK' => 'kr',
			'JEP' => '£',
			'JMD' => 'J$',
			'JOD' => 'JD',
			'JPY' => '¥',
			'KES' => 'KSh',
			'KGS' => 'лв',
			'KHR' => '៛',
			'KMF' => 'CF',
			'KPW' => '₩',
			'KRW' => '₩',
			'KWD' => 'د.ك',
			'KYD' => '$',
			'KZT' => '₸',
			'LAK' => '₭',
			'LBP' => '£',
			'LKR' => '₨',
			'LRD' => '$',
			'LSL' => 'L',
			'LTL' => 'Lt',
			'LVL' => 'Ls',
			'LYD' => 'ل.د',
			'MAD' => 'د.م.',
			'MDL' => 'L',
			'MGA' => 'Ar',
			'MKD' => 'ден',
			'MMK' => 'K',
			'MNT' => '₮',
			'MOP' => 'MOP$',
			'MRO' => 'UM',
			'MUR' => '₨',
			'MVR' => '.ރ',
			'MWK' => 'MK',
			'MXN' => '$',
			'MYR' => 'RM',
			'MZN' => 'MT',
			'NAD' => '$',
			'NGN' => '₦',
			'NIO' => 'C$',
			'NOK' => 'kr',
			'NPR' => '₨',
			'NZD' => '$',
			'OMR' => '﷼',
			'PAB' => 'B/.',
			'PEN' => 'S/.',
			'PGK' => 'K',
			'PHP' => '₱',
			'PKR' => '₨',
			'PLN' => 'zł',
			'PYG' => 'Gs',
			'QAR' => '﷼',
			'RON' => 'lei',
			'RSD' => 'Дин.',
			'RUB' => '₽',
			'RWF' => 'ر.س',
			'SAR' => '﷼',
			'SBD' => '$',
			'SCR' => '₨',
			'SDG' => '£',
			'SEK' => 'kr',
			'SGD' => '$',
			'SHP' => '£',
			'SLL' => 'Le',
			'SOS' => 'S',
			'SRD' => '$',
			'STD' => 'Db',
			'SVC' => '$',
			'SYP' => '£',
			'SZL' => 'L',
			'THB' => '฿',
			'TJS' => 'TJS',
			'TMT' => 'm',
			'TND' => 'د.ت',
			'TOP' => 'T$',
			'TRY' => '₤',
			'TTD' => '$',
			'TWD' => 'NT$',
			'TZS' => 'TSh',
			'UAH' => '₴',
			'UGX' => 'USh',
			'USD' => '$',
			'UYU' => '$U',
			'UZS' => 'лв',
			'VEF' => 'Bs',
			'VND' => '₫',
			'VUV' => 'VT',
			'WST' => 'WS$',
			'XAF' => 'FCFA',
			'XCD' => '$',
			'XDR' => 'SDR',
			'XOF' => 'FCFA',
			'XPF' => 'F',
			'YER' => '﷼',
			'ZAR' => 'R',
			'ZMK' => 'ZK',
			'ZWL' => 'Z$',
			'$' => '$',
		];
		if( isset($currencySymbols[$country_code]) ){
			return $currencySymbols[$country_code];
		}
		return '';
	}


	/**
	 * Redirects LearnDash Funnel templates
	 * based on course access
	 *
	 * @param $redirect_url
	 * @param false $payment_method
	 */
	public static function learndash_template_safe_redirect( $redirect_url, $payment_method = false )
	{
		if( $payment_method ) {
			$redirect_url = add_query_arg( 'wpfnl_ld_payment', $payment_method, $redirect_url );
		}
		exit( wp_safe_redirect( $redirect_url ) );
	}


	/**
	 * Change lms order meta value
	 *
	 * @param $funnel_id, $step_id, $user_id
	 */
	public static function update_lms_order_status( $funnel_id, $step_id, $user_id ){

		if( $funnel_id && $step_id && $user_id ){
			global $wpdb;
			$wpfnl_lms_order_table = $wpdb->prefix . 'wpfnl_lms_order';
			$query = $wpdb->prepare("UPDATE {$wpfnl_lms_order_table} SET meta='order' WHERE funnel_id = %d AND step_id = %d AND user_id = %d", $funnel_id, $step_id, $user_id);
			$wpdb->query( $query );

		}

	}



	/**
	 * WPFunnels create LMS order table
	 *
	 *@param NULL
	 *@return NULL
	 */
	public static function create_lms_order_table() {
		global $wpdb;
		$table_name = $wpdb->prefix . "wpfnl_lms_order";
		$charset_collate = $wpdb->get_charset_collate();
		$sql = "CREATE TABLE IF NOT EXISTS $table_name (
				id BIGINT(20) NOT NULL AUTO_INCREMENT,
				funnel_id BIGINT(20),
				step_id BIGINT(20),
				user_id BIGINT(20) DEFAULT 0,
				meta TEXT NULL,
				payment_method TEXT NULL,
				amount TEXT NULL,
				step_type TEXT NULL,
				course_id BIGINT(20) NULL,
				order_data LONGTEXT NULL,
				date_created DATETIME DEFAULT '0000-00-00 00:00:00',
				PRIMARY KEY (id)
			  ) $charset_collate; ";

		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
		dbDelta($sql);
	}


		/**
     * get webhook data for sending
     * 
     */
    public static function get_webhook_data( $offer_product ){
		
		if( isset($offer_product['id']) && $offer_product['id'] ){
			$funnel_id = get_post_meta($offer_product['step_id'],'_funnel_id', true);
            $type = get_post_meta($funnel_id,'_wpfnl_funnel_type', true);

            if( 'lms' === $type ){
				$course_data = self::get_course_details_by_id( $offer_product['id'] );
				$formatted_body['Product ID']       = $course_data['id'];
				$formatted_body['Product Name']     = $course_data['title'];
				$formatted_body['Product Price']    = $course_data['price'];
				$formatted_body['Payment Method']   = isset($_GET['wpfnl_ld_payment']) && $_GET['wpfnl_ld_payment'] ? $_GET['wpfnl_ld_payment'] : '';
				$cookie_name        = 'wpfunnels_automation_data';
        		$cookie             = isset( $_COOKIE[$cookie_name] ) ? json_decode( wp_unslash( $_COOKIE[$cookie_name] ), true ) : array();
				if( isset($cookie['user_info']) ){
					$formatted_body['Customer Name']    = isset($cookie['user_info']['first_name']) &&  isset($cookie['user_info']['last_name']) ? $cookie['user_info']['first_name'].' '.$cookie['user_info']['last_name'] : '';
					$formatted_body['Customer Email']   = isset($cookie['user_info']['email']) ? $cookie['user_info']['email'] : '';
				
				}
				return $formatted_body;
			}
			
		}
		return [];
    } 


	/**
     * get selected webhook data for sending
     * 
     */
    public static function get_webhook_selected_data( $offer_product ){
		
		if( isset($offer_product['id']) && $offer_product['id'] ){
			$funnel_id = get_post_meta($offer_product['step_id'],'_funnel_id', true);
            $type = get_post_meta($funnel_id,'_wpfnl_funnel_type', true);

            if( 'lms' === $type ){
				$course_data = self::get_course_details_by_id( $offer_product['id'] );
				$formatted_body['product_id']       = $course_data['id'];
				$formatted_body['product_name']     = $course_data['title'];
				$formatted_body['product_price']    = $course_data['price'];
				$formatted_body['payment_method']   = isset($_GET['wpfnl_ld_payment']) && $_GET['wpfnl_ld_payment'] ? $_GET['wpfnl_ld_payment'] : '';
				$cookie_name        = 'wpfunnels_automation_data';
        		$cookie             = isset( $_COOKIE[$cookie_name] ) ? json_decode( wp_unslash( $_COOKIE[$cookie_name] ), true ) : array();
				if( isset($cookie['user_info']) ){
					$formatted_body['customer_name']    = isset($cookie['user_info']['first_name']) &&  isset($cookie['user_info']['last_name']) ? $cookie['user_info']['first_name'].' '.$cookie['user_info']['last_name'] : '';
					$formatted_body['customer_email']   = isset($cookie['user_info']['email']) ? $cookie['user_info']['email'] : '';
				
				}
				return $formatted_body;
			}
			
		}
		return [];
    }


}