<?php
namespace WPFunnels\Controller;
use WPFunnels\Controller\Wpfnl_Controller_Type;
use WPFunnels\Wpfnl_functions;
use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;


class Wpfnl_Controller_Lms extends Wpfnl_Controller_Type
{
    
    /**
     * Get courses
     * 
     * @param $term
     * @return Array
     */
    public function get_items( $step_id = '' ){
        $response = [];
        if( $step_id ){
            $products 			=  get_post_meta($step_id, '_wpfnl_checkout_products', true);
            $discount 			=  get_post_meta($step_id, '_wpfnl_checkout_discount_main_product', true);
            
            if( $products && Wpfnl_lms_learndash_functions::is_learndash_active()){
                $course   = Wpfnl_lms_learndash_functions::get_course_by_id( $products[ 0 ][ 'id' ] );
                $course_details   = Wpfnl_lms_learndash_functions::get_course_details_by_id( $products[ 0 ][ 'id' ] );
                $price    = function_exists( 'learndash_get_course_price' ) ? learndash_get_course_price( $course ) : '';
                $price	  = isset( $price['price'] ) ? $price['price'] : $price;
                $image_id = get_post_meta( $products[ 0 ][ 'id' ], '_thumbnail_id', true );
                $image    = wp_get_attachment_image_src( $image_id, 'single-post-thumbnail' );

                $response['products'][] = array(
                    'id' 				=> $products[0]['id'],
                    'title' 			=> $course->post_title,
                    'price' 			=> $price,
                    'numeric_price'     => $price,
                    'no_of_lession'     => $course_details['no_of_lesson'],
                    'type'     			=> $course_details['type'],
                    'currency' 			=> function_exists( 'learndash_get_currency_symbol' ) ? Wpfnl_lms_learndash_functions::get_currency_symbol_from_country_code(learndash_get_currency_symbol()) : Wpfnl_lms_learndash_functions::get_currency_symbol_from_country_code(learndash_30_get_currency_symbol()),
                    'sale_price' 		=> $price,
                    'regular_price' 	=> $price,
                    'quantity' 			=> 1,
                    'image' 			=> isset( $image[0] ) ? $image[0] : '',
                    'discount_type' 	=> '',
                    'discount_value' 	=> '',
                    'product_edit_link' => get_edit_post_link($products[0]['id']),
                    'product_view_link' => get_permalink($products[0]['id']),
                );
                $response['coupon'] = '';
                $response['isMultipleProduct'] = '';
                $response['isQuantity'] = '';
                $response['discount'] = $discount;
            }else{
                $response['success'] = true;
                $response['products'] = [];
                $response['coupon'] = '';
                $response['isMultipleProduct'] = '';
                $response['isQuantity'] = '';
                $response['discount'] = $discount;
            }
        }
      
        return $response;
    }


    /**
     * get order bump settings for lms
     * 
     * @param Array all_settings
     * @return Array
     */
    public function get_ob_settings( $all_settings ){

        if( $all_settings && Wpfnl_lms_learndash_functions::is_learndash_active() ){
            foreach( $all_settings as $key=>$settings ){
                $course_meta = Wpfnl_lms_learndash_functions::get_course_details_by_id($all_settings[$key]['product']);
                $all_settings[$key]['discountOption'] = 'original';
                $all_settings[$key]['quantity'] = '1';
                $all_settings[$key]['discountOption'] = 'original';
                $all_settings[$key]['price'] = $course_meta['price'];
                $all_settings[$key]['htmlPrice'] = '';

                if( isset( $all_settings[$key]['productDescriptionText']) ){
                    $all_settings[$key]['productDescriptionText'] = strip_tags($all_settings[$key]['productDescriptionText']);
                }
                if( !isset($all_settings[$key]['productSearchName']) ){
                    $all_settings[$key]['productSearchName'] = isset($all_settings[$key]['productName']) ? $all_settings[$key]['productName'] : '';
                }
            }
            return $all_settings;
        }
        return [];
    }


    /**
     * Update order bump settings for lms
     * 
     * @param
     * @return
     */
    public function update_ob_settings( $all_settings ){
        if( $all_settings && Wpfnl_lms_learndash_functions::is_learndash_active() ){
            foreach( $all_settings as $key=>$settings ){

                $all_settings[$key]['discountOption'] = 'original';
                $all_settings[$key]['quantity'] = '1';
                $all_settings[$key]['discountOption'] = 'original';
                $all_settings[$key]['price'] = '';
                $all_settings[$key]['htmlPrice'] = '';
                
            }
            return $all_settings;
        }
        return [];
    }

}