<?php
/**
 * Order details shortcode class
 */
namespace WPFunnels\LMS\Shortcodes;

use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;
use WPFunnels\Meta\Wpfnl_Default_Meta;
use WPFunnels\Wpfnl_functions;

/**
 * Class WC_Shortcode_Optin
 * @package WPFunnels\Shortcodes
 */
class Wpfnl_Shortcode_LMS_Checkout {

	/**
	 * Attributes
	 *
	 * @var array
	 */
	protected $attributes = array();


	/**
	 * Wpfnl_Shortcode_Order_details constructor.
	 * @param array $attributes
	 */
	public function __construct( $attributes = array() ) {

		$this->attributes = $this->parse_attributes( $attributes );


	}


	/**
	 * Get shortcode attributes.
	 *
	 * @since  3.2.0
	 * @return array
	 */
	public function get_attributes() {
		return $this->attributes;
	}


	/**
	 * parse attributes
	 *
	 * @param $attributes
	 * @return array
	 */
	protected function parse_attributes( $attributes ) {

		$attributes = shortcode_atts(
			array(
				'lms_order_header_title' 		            => 'Order Details',
				'lms_order_header_title_color' 		        => '',
				'lms_order_header_title_size' 		        => '',
				'lms_order_header_background_color' 		=> '',

				'lms_order_course_details_title' 			=> 'Course Details',
				'lms_order_course_info_title_color'         => '',
				'lms_order_course_info_title_size'          => '',
				'lms_order_course_details_background_color' => '',

                'lms_order_course_description'		        => 'Briefly describe your course in 2-3 lines.',
                'lms_order_course_info_description_color'   => '',
                'lms_order_course_info_description_size'    => '',

                'lms_price_color'                           => '',
                'lms_footer_title_color'                    => '',
                'lms_footer_title_size'                     => '',
                'lms_footer_background_color'               => '',

				'lms_order_plan_details_title'			    => 'Plan Details',
				'class'					                    => '',
                'btn_font_size'                             => '',
                'btn_margin'                                => '',
                'btn_padding'                               => '',
                'btn_background_color'                      => '',
                'btn_color'                                 => '',
                'btn_radius'                                => '',
			),
			$attributes
		);
		return $attributes;
	}

	/**
	 * get wrapper classes
	 *
	 * @return array
	 */
	protected function get_wrapper_classes() {
		$classes = array( 'wpfnl', 'wpfnl-order-details-wrapper' );
		return $classes;
	}


	/**
	 * content of check form
	 *
	 * @return string
	 */
	public function get_content() {
        $data        = \WPFunnels\Wpfnl_functions::get_sanitized_get_post();
        $step_id 	= isset($data['post']['current_page']['id']) ? $data['post']['current_page']['id'] : get_the_ID();
        $step_type = get_post_meta($step_id,'_step_type',true);
        $funnel_id = get_post_meta($step_id,'_funnel_id',true);
        $get_learn_dash_setting = Wpfnl_lms_learndash_functions::get_learndash_settings($funnel_id);
        if ($step_type != 'checkout'){
            return __('Sorry, Please place the element in WPFunnels Checkout page');
        }else{
            if($get_learn_dash_setting == 'yes'){
                $course = Wpfnl_lms_learndash_functions::get_course_details($step_id);

                if (!empty($course)){
                    ob_start();
                    $course_title 		= isset($course['title']) ? $course['title']: '';
                    $description 		= isset($course['description']) ? $course['description']: '';
                    $no_of_lesson 		= isset($course['no_of_lesson']) ? $course['no_of_lesson']: '';
                    $course_type 		= isset($course['type']) ? $course['type']: '';
                    $billing_cycle 		= isset($course['billing_cycle']) ? $course['billing_cycle']: '';
                    $recurring_time 	= isset($course['recurring_time']) ? $course['recurring_time']: '';
                    $price 				= isset($course['price'])? $course['price']: '';
                    $trial_price 		= isset($course['trial_price']) ? $course['trial_price']: '';
                    $trial_period 		= isset($course['trial_period']) ? $course['trial_period']: '';
                    $is_expire 			= isset($course['is_expire']) ? $course['is_expire']: '';
                    $expire_days 		= isset($course['expire_days']) ? $course['expire_days']: '';
                    $image 				= isset($course['image']) ? $course['image']: '';
                    $billing_cycle_unit = isset($course['billing_cycle_unit']) ? $course['billing_cycle_unit']: '';
                    $trial_period_unit  = isset($course['trial_period_unit']) ? $course['trial_period_unit']: '';
                    $currency           = isset($course['currency']) ? $course['currency']: '';
                    $discount_price  	= isset($course['discount_price']) ? $course['discount_price'] : '';
                    //----course type------
                    if( $course_type == 'subscribe' ) {
                        $course_type_text = 'Subscription';

                    }else if( $course_type == 'open' || $course_type == 'free' ){
                        $course_type_text = 'Free';

                    }else if( $course_type == 'paynow' ){
                        $course_type_text = 'One-time Payment';

                    }else {
                        $course_type_text = 'Closed';
                    }

                    //----billing cycle unit------
                    if( $course['billing_cycle_unit'] == 'Y' ) {
                        $billing_cycle_unit_text = 'year';

                    }else if( $course['billing_cycle_unit'] == 'M' ){
                        $billing_cycle_unit_text = 'month';

                    }else if( $course['billing_cycle_unit'] == 'W' ){
                        $billing_cycle_unit_text = 'week';

                    }else {
                        $billing_cycle_unit_text = 'day';
                    }

                    //----trial period unit------
                    if( $course['trial_period_unit'] == 'Y' ) {
                        $trial_period_unit_text = 'year';

                    }else if( $course['trial_period_unit'] == 'M' ){
                        $trial_period_unit_text = 'month';

                    }else if( $course['trial_period_unit'] == 'W' ){
                        $trial_period_unit_text = 'week';

                    }else {
                        $trial_period_unit_text = 'day';
                    }
                    do_action( 'wpfunnels/before_checkout_form', $step_id );
                    $lms_order_header_style = 'font-size:'.$this->attributes['lms_order_header_title_size'].'; color: '.$this->attributes['lms_order_header_title_color'].'';
                    $lms_order_info_style   = 'font-size:'.$this->attributes['lms_order_course_info_title_size'].'; color: '.$this->attributes['lms_order_course_info_title_color'].'';
                    $lms_order_info_description_style   = 'font-size:'.$this->attributes['lms_order_course_info_description_size'].'; color: '.$this->attributes['lms_order_course_info_description_color'].'';
                    $lms_order_footer_style   = 'font-size:'.$this->attributes['lms_footer_title_size'].'; color: '.$this->attributes['lms_footer_title_color'].'';
                    $button_style = '';
                    $button_custom_style = '';
                    if ('' != $this->attributes['btn_font_size'] || '' != $this->attributes['btn_margin'] || '' != $this->attributes['btn_padding']
                        || '' != $this->attributes['btn_background_color'] || '' != $this->attributes['btn_color'] || '' != $this->attributes['btn_radius']){
                        $button_style = ' style="font-size:'.$this->attributes['btn_font_size'].'; margin: '.$this->attributes['btn_margin'].'; padding: '.$this->attributes['btn_padding'].'; 
                                    background-color: '.$this->attributes['btn_background_color'].'; color: '.$this->attributes['btn_color'].'; border-radius: '.$this->attributes['btn_radius'].';"';
                        $button_custom_style = 'font-size:'.$this->attributes['btn_font_size'].'!important; margin: '.$this->attributes['btn_margin'].'!important; padding: '.$this->attributes['btn_padding'].'!important; 
                                    background-color: '.$this->attributes['btn_background_color'].'!important; color: '.$this->attributes['btn_color'].'!important; border-radius: '.$this->attributes['btn_radius'].'!important;';
                    }
                    ?>
                    <div>
                        <?php  do_action('wpfunnels/before_lms_order_deatils'); ?>
                    </div>

                    <div class="wpfnl-reset wpfnl-lms-checkout <?php echo $this->attributes['class'] ?>">
                        <div class="lms-checkout-box">
                            <div class="lms-checkout-header" style="background-color: <?php echo $this->attributes['lms_order_header_background_color'] ?>">
                                <h4 style="<?php echo $lms_order_header_style ?>"><?php echo $this->attributes['lms_order_header_title'] ?></h4>
                            </div>
                            <div class="lms-checkout-body" style="background-color: <?php echo $this->attributes['lms_order_course_details_background_color'] ?>">
                                <div class="course-image">
                                    <img src="<?php echo $image; ?>" alt="course image" />
                                </div>

                                <div class="course-content">
                                    <h2 class="course-title" style="<?php echo $lms_order_info_style ?>">
                                        <?php echo $course_title ?>
                                        <span class="no-of-lesson" style="<?php echo $lms_order_info_description_style ?>">- <?php echo $no_of_lesson ?> Lessons</span>
                                    </h2>

                                    <div class="lms-single-block course-details-block">
                                        <h4 class="lms-block-title" style="<?php echo $lms_order_info_style ?>"><?php echo $this->attributes['lms_order_course_details_title'] ?></h4>
                                        <div class="course-description" style="<?php echo $lms_order_info_description_style ?>">
                                            <?php
                                            echo $this->attributes['lms_order_course_description'];
                                            ?>
                                        </div>
                                    </div>

                                    <div class="lms-single-block course-plan-block">
                                        <h4 class="lms-block-title" style="<?php echo $lms_order_info_style ?>"><?php echo $this->attributes['lms_order_plan_details_title']; ?></h4>
                                        <div class="course-plan" style="<?php echo $lms_order_info_description_style ?>">
                                            <?php
                                            if( $course_type == 'subscribe' ){

                                                if( $discount_price ){
                                                    echo $course_type_text .' - <span class="primary-color" style="color: '.$this->attributes['lms_price_color'].'"><del>'.$currency.$price.'</del> <ins style="text-decoration:none">'.$currency.$discount_price.'</ins>/'.$billing_cycle_unit_text.'</span>';
                                                }else{
                                                    echo $course_type_text .' - <span class="primary-color" style="color: '.$this->attributes['lms_price_color'].'">'.$currency.$price.'/'.$billing_cycle_unit_text.'</span>';
                                                }

                                                if( !empty( $trial_period ) ){
                                                    echo '<small> ('.$trial_period.' '.$trial_period_unit_text.' trial period)</small>';
                                                }

                                            } else if( $course_type == 'paynow' ){

                                                if( $discount_price ){
                                                    echo $course_type_text .' - <span class="primary-color" style="color: '.$this->attributes['lms_price_color'].'"><del>'.$currency.$price.'</del>  <ins style="text-decoration:none">'.$currency.$discount_price.'</ins></span>';
                                                }else{
                                                    echo $course_type_text .' - <span class="primary-color" style="color: '.$this->attributes['lms_price_color'].'">'.$currency.$price.'</span>';
                                                }

                                                if( $is_expire == 'on' ){
                                                    echo '<small> (expires in '.$expire_days.' days)</small>';
                                                }
                                            }else{
                                                echo $course_type_text;

                                                if( $is_expire == 'on' ){
                                                    echo '<small> (Login or register to enroll) (expires in '.$expire_days.' days)</small>';
                                                }
                                            }
                                            ?>
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </div>
                        <div class="lms-checkout-footer" style="background-color:<?php echo $this->attributes['lms_footer_background_color'] ?>">
                            <?php
                            if( $course_type == 'subscribe' ){
                                if( !empty( $trial_period ) ){
                                    ?>

                                    <p class="footer-title">Course Trial Fees <span class="footer-price" style="color: <?php echo $this->attributes['lms_price_color'] ?>"><?php echo $currency.$trial_price; ?></span></p>

                                <?php }else{ ?>

                                    <p class="footer-title">Course Subscription Fees <span class="footer-price" style="color: <?php echo $this->attributes['lms_price_color'] ?>"><?php echo $currency.$price; ?></span></p>
                                    <?php
                                }
                            }else if( $course_type == 'paynow' ){
                                if( $discount_price ){ ?>
                                    <p class="footer-title" style="<?php echo $lms_order_footer_style ?>">Course Fees <span class="footer-price" style="color: <?php echo $this->attributes['lms_price_color'] ?>"><del><?php echo $currency.$price; ?></del> <ins style="text-decoration:none"> <?php echo $currency.$discount_price; ?> </ins></span></p>
                                <?php } else{ ?>
                                    <p class="footer-title" style="<?php echo $lms_order_footer_style ?>">Course Fees <span class="footer-price" style="color: <?php echo $this->attributes['lms_price_color'] ?>"><?php echo $currency.$price; ?></span></p>
                                <?php }
                                ?>

                            <?php }else{ ?>
                                <p class="footer-title" style="<?php echo $lms_order_footer_style ?>">Course Fees <span class="footer-price" style="color: <?php echo $this->attributes['lms_price_color'] ?>">Free</span></p>
                            <?php } ?>

                            <!-- buy now button -->
                            <?php
                            if( is_user_logged_in() ) {
                                $course_access   = sfwd_lms_has_access( $course[ 'id' ], get_current_user_id() );
                                $next_step_url   = Wpfnl_lms_learndash_functions::get_next_step_url( $funnel_id, $step_id ) . '?wpfnl_ld_payment=free&wpfnl_course_status=enrolled';
                                $lms_button_text = get_option( 'learndash_settings_custom_labels' );
                                $button_text     = !empty( $lms_button_text[ 'button_take_this_course' ] ) ? esc_html( $lms_button_text[ 'button_take_this_course' ] ) : __( 'Take This Course', 'wpfunnels-pro-lms' );

                                if( $course_access ) {
                                    echo "<a class='btn-default' href='{$next_step_url}' id='wpfnl-lms-access-course'>{$button_text}</a>";
                                    echo '<span class="wpfnl-lms-access-course-message"></span>';
                                }
                                else if( 'free' === $course_type ) {
                                    $next_step_url = Wpfnl_lms_learndash_functions::get_next_step_url( $funnel_id, $step_id );
                                    $user_id       = get_current_user_id();
                                    echo "<a class='btn-default' href='{$next_step_url}' data-user_id='{$user_id}' data-step_id='{$step_id}' data-course_id='{$course['id']}' id='wpfnl-lms-free-course'>{$button_text}</a>";
                                    echo '<span class="wpfnl-lms-free-course-message"></span>';
                                }
                                else {
                                    echo $this->button_style( $button_custom_style );
                                    echo do_shortcode( "[learndash_payment_buttons course_id='{$course['id']}']" );
                                }
                            }
                            else {
                                echo $this->button_style( $button_custom_style );
                                echo do_shortcode( '[learndash_login]' );
                            }
                            ?>
                        </div>

                    </div>

                    <div>
                        <?php  do_action('wpfunnels/after_lms_order_deatils'); ?>
                    </div>
                    <?php
                    return ob_get_clean();


                }else{
                    return __("Please Add Course in checkout");
                }
            }else{
                return __('Sorry, Please place the element in WPFunnels when learnDash is active ');
            }
        }
	}
    public function button_style($button_custom_style){
        return '
        <style>
        .learndash-wrapper .ld-login-button.ld-button,
        .learndash-wrapper .btn-join{
           '.$button_custom_style.' 
        }
        </style>
        
        ';
    }


}
