<?php
/**
 * Order details shortcode class
 */
namespace WPFunnels\LMS\Shortcodes;

use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;
use WPFunnels\Meta\Wpfnl_Default_Meta;
use WPFunnels\Wpfnl_functions;

/**
 * Class WC_Shortcode_Optin
 * @package WPFunnels\Shortcodes
 */
class Wpfnl_Shortcode_LMS_Offer_button {

    /**
     * Attributes
     *
     * @var array
     */
    protected $attributes = array();


    /**
     * Wpfnl_Shortcode_Order_details constructor.
     * @param array $attributes
     */
    public function __construct( $attributes = array() ) {

        $this->attributes = $this->parse_attributes( $attributes );


    }


    /**
     * Get shortcode attributes.
     *
     * @since  3.2.0
     * @return array
     */
    public function get_attributes() {
        return $this->attributes;
    }


    /**
     * parse attributes
     *
     * @param $attributes
     * @return array
     */
    protected function parse_attributes( $attributes ) {

        $attributes = shortcode_atts(
            array(
                'button_text' 		            => 'Take this course',
                'button_action' 			    => 'accept',
                'button_type'		            => 'upsell',
                'button_alignment'		        => 'left',
                'class'					        => '',
                'btn_font_size'                 => '',
                'btn_margin'                    => '',
                'btn_padding'                   => '',
                'btn_background_color'          => '',
                'btn_color'                     => '',
                'btn_radius'                    => '',
            ),
            $attributes
        );
        return $attributes;
    }

    /**
     * get wrapper classes
     *
     * @return array
     */
    protected function get_wrapper_classes() {
        $classes = array( 'wpfnl', 'wpfnl-order-details-wrapper' );
        return $classes;
    }


    /**
     * content of check form
     *
     * @return string
     */
    public function get_content() {
        $data        = \WPFunnels\Wpfnl_functions::get_sanitized_get_post();
        $step_id 	= isset($data['post']['current_page']['id']) ? $data['post']['current_page']['id'] : get_the_ID();
        ob_start();
        $button_style = '';
        $button_custom_style = '';
        if ('' != $this->attributes['btn_font_size'] || '' != $this->attributes['btn_margin'] || '' != $this->attributes['btn_padding']
            || '' != $this->attributes['btn_background_color'] || '' != $this->attributes['btn_color'] || '' != $this->attributes['btn_radius']){
            $button_style = ' style="font-size:'.$this->attributes['btn_font_size'].'; margin: '.$this->attributes['btn_margin'].'; padding: '.$this->attributes['btn_padding'].'; 
                                    background-color: '.$this->attributes['btn_background_color'].'; color: '.$this->attributes['btn_color'].'; border-radius: '.$this->attributes['btn_radius'].';"';
            $button_custom_style = 'font-size:'.$this->attributes['btn_font_size'].'!important; margin: '.$this->attributes['btn_margin'].'!important; padding: '.$this->attributes['btn_padding'].'!important; 
                                    background-color: '.$this->attributes['btn_background_color'].'!important; color: '.$this->attributes['btn_color'].'!important; border-radius: '.$this->attributes['btn_radius'].'!important;';
        }
        $funnel_id = get_post_meta($step_id,'_funnel_id',true);
        $get_learn_dash_setting = Wpfnl_lms_learndash_functions::get_learndash_settings($funnel_id);
        $lms_button_text = get_option( 'learndash_settings_custom_labels' );
        if ($get_learn_dash_setting == 'yes'){
            $step_type = get_post_meta($step_id, '_step_type', true);
            if ($step_type != 'upsell' && $step_type != 'downsell'){
                echo __('Sorry, Please place the element in WPFunnels Offer page');
            }else{
                $button_type           = $this->attributes['button_type'];
                $button_action         = $this->attributes['button_action'];
                $button_id =  'wpfunnels_'.$button_type.'_'.$button_action;

                if ($button_action == 'accept'){
                    $button_text = !empty($lms_button_text['button_take_this_course']) ? $lms_button_text['button_take_this_course'] : $this->attributes['button_text'];
                    $lms_product_id = get_post_meta($step_id,'_wpfnl_upsell_products',true);
                    if($step_type == 'downsell'){
                        $lms_product_id = get_post_meta($step_id,'_wpfnl_downsell_products',true);
                    }
                    $course_access = sfwd_lms_has_access( $lms_product_id[0]['id'], get_current_user_id() );
                    $next_step_url = Wpfnl_lms_learndash_functions::get_next_step_url($funnel_id,$step_id).'?wpfnl_ld_payment=free';
                    if ($course_access ){
                        echo '<a class="btn-default btn-join '.$this->attributes['class'].'" href="'.$next_step_url.'" id="wpfnl-lms-access-course" '.$button_style.'>'.$button_text.'</a>';
                        echo '<span class="wpfnl-lms-access-course-message"></span>';
                    }else{
                        $course = isset($lms_product_id[0]['id']) ? Wpfnl_lms_learndash_functions::get_course_details_by_id($lms_product_id[0]['id']) : [];
                        if( !empty($course) ){
                            if( isset($course['type'])  && $course['type'] == 'free' ){
                                $next_step_url = Wpfnl_lms_learndash_functions::get_next_step_url($funnel_id,$step_id);
                                echo '<a class="btn-default '.$this->attributes['class'].'" href="'.$next_step_url.'" data-user_id="'.get_current_user_id().'" data-step_id="'.$step_id.'" data-course_id="'.$course['id'].'" id="wpfnl-lms-free-course" '.$button_style.'>'.$button_text.'</a>';
                                echo '<span class="wpfnl-lms-free-course-message"></span>';
                            }
                            else {
                                echo $this->button_style( $button_custom_style );
                                echo do_shortcode( "[learndash_payment_buttons course_id='{$course['id']}']" );
                            }
                        }
                    }
                }
                else{
                    ?>
                    <div class="oxy-offer-button-<?php echo $this->attributes['button_alignment']; ?>">
                        <a class="btn-default btn-join wpfnl-oxy-offer-btn offer-button wpfunnels_offer_button <?php echo $this->attributes['class'] ?>"  data-offertype="<?php echo $button_type ?>" id="<?php echo $button_id ?>  <?php echo $button_style ?>"> <?php echo $this->attributes['button_text'] ?> </a>
                    </div>
                    <?php
                }
            }
        }else{
            echo __('Sorry, Please place the element in WPFunnels when learnDash is active ');
        }



        return ob_get_clean();
    }


    public function button_style($button_custom_style){
        return '
        <style>
        .learndash-wrapper .btn-join{
           '.$button_custom_style.' 
        }
        </style>
        
        ';
    }


}
