<?php
/**
 * Order details shortcode class
 */
namespace WPFunnels\LMS\Shortcodes;

use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;
use WPFunnels\Meta\Wpfnl_Default_Meta;
use WPFunnels\Wpfnl_functions;

/**
 * Class WC_Shortcode_Optin
 * @package WPFunnels\Shortcodes
 */
class Wpfnl_Shortcode_LMS_Order_Details {

    /**
     * Attributes
     *
     * @var array
     */
    protected $attributes = array();


    /**
     * Wpfnl_Shortcode_Order_details constructor.
     * @param array $attributes
     */
    public function __construct( $attributes = array() ) {

        $this->attributes = $this->parse_attributes( $attributes );


    }


    /**
     * Get shortcode attributes.
     *
     * @since  3.2.0
     * @return array
     */
    public function get_attributes() {
        return $this->attributes;
    }


    /**
     * parse attributes
     *
     * @param $attributes
     * @return array
     */
    protected function parse_attributes( $attributes ) {

        $attributes = shortcode_atts(
            array(
                'enable_course_review' 		            => 'on',
                'enable_payment_method' 			    => 'on',
                'enable_course_details'		            => 'on',
                'class'					                => ''
            ),
            $attributes
        );
        return $attributes;
    }

    /**
     * get wrapper classes
     *
     * @return array
     */
    protected function get_wrapper_classes() {
        $classes = array( 'wpfnl', 'wpfnl-order-details-wrapper' );
        return $classes;
    }


    /**
     * content of check form
     *
     * @return string
     */
    public function get_content() {
        $data        = \WPFunnels\Wpfnl_functions::get_sanitized_get_post();
        $step_id 	= isset($data['post']['current_page']['id']) ? $data['post']['current_page']['id'] : get_the_ID();
        $step_type = get_post_meta($step_id,'_step_type',true);
        ob_start();
        if (!$step_type){
            echo __('Sorry, Please place the element in WPFunnels Thank You page');
        }else{
            $funnel_id = get_post_meta($step_id,'_funnel_id',true);
            $get_learn_dash_setting = Wpfnl_lms_learndash_functions::get_learndash_settings($funnel_id);
            if($get_learn_dash_setting == 'yes'){
                $order_details = Wpfnl_lms_learndash_functions::get_lms_order_details( get_current_user_id() ,$funnel_id );
                if (!empty($order_details)){
                    foreach($order_details as $details){
                        $course_details =  $details['course_details'];
                        $payment_method =  $details['payment_method'];

                        $course_type 		= $course_details['type'];
                        $billing_cycle 		= $course_details['billing_cycle'];
                        $recurring_time 	= $course_details['recurring_time'];
                        $price 				= $course_details['price'];
                        $trial_price 		= $course_details['trial_price'];
                        $trial_period 		= $course_details['trial_period'];
                        $is_expire 			= $course_details['is_expire'];
                        $expire_days 		= $course_details['expire_days'];
                        $currency           = $course_details['currency'];
                        $price  			= isset($details['amount']) ? $details['amount'] : $price;

                        //----course type------
                        if( $course_type == 'subscribe' ) {
                            $course_type_text = 'Subscription';

                        }else if( $course_type == 'open' || $course_type == 'free' ){
                            $course_type_text = 'Free';

                        }else if( $course_type == 'paynow' ){
                            $course_type_text = 'One-time Payment';

                        }else {
                            $course_type_text = 'Closed';
                        }

                        //----billing cycle unit------
                        if( $course_details['billing_cycle_unit'] == 'Y' ) {
                            $billing_cycle_unit_text = 'year';

                        }else if( $course_details['billing_cycle_unit'] == 'M' ){
                            $billing_cycle_unit_text = 'month';

                        }else if( $course_details['billing_cycle_unit'] == 'W' ){
                            $billing_cycle_unit_text = 'week';

                        }else {
                            $billing_cycle_unit_text = 'day';
                        }

                        //----trial period unit------
                        if( $course_details['trial_period_unit'] == 'Y' ) {
                            $trial_period_unit_text = 'year';

                        }else if( $course_details['trial_period_unit'] == 'M' ){
                            $trial_period_unit_text = 'month';

                        }else if( $course_details['trial_period_unit'] == 'W' ){
                            $trial_period_unit_text = 'week';

                        }else {
                            $trial_period_unit_text = 'day';
                        }

                        ?>
                        <div class="lms-order-details <?php echo $this->attributes['class'] ?>">
                            <!-- <h2 class="order-details-title"><span>Thank you.</span> You have been enrolled to your courses.</h2> -->

                            <div class="order-details-table">
                                <div class="order-item order-tbl-header">
                                    <?php
                                    if( $details['step_type'] == 'upsell' || $details['step_type'] == 'downsell' ){
                                        ?>
                                        <span class="course-name">Course Offer</span>
                                        <?php
                                    }else{
                                        ?>
                                        <span class="course-name">Course</span>
                                        <?php
                                    }
                                    ?>


                                    <span class="price-total">Total</span>
                                </div>

                                <?php if($this->attributes['enable_course_review'] == 'on') {?>
                                    <div class="order-item course-info">
									<span class="course-name">
										<span><?php echo $course_details['title']; ?></span>
										<a href="<?php echo get_post_permalink($course_details['id']) ?>">View Course</a>
									</span>

                                        <span class="price-total">

									<?php
                                    if( $course_type == 'subscribe' ){
                                        echo  $currency.$price.'/'.$billing_cycle_unit_text;

                                    } else if( $course_type == 'paynow' ){
                                        echo $currency.$price;
                                    }else{
                                        echo $course_type_text;
                                    }
                                    ?>


                                    <?php //echo $course_details['currency'].$course_details['price']; ?>
								</span>
                                    </div>
                                <?php } ?>

                                <?php if($this->attributes['enable_payment_method'] == 'on') {?>
                                    <div class="order-item payment-info">
								<span class="course-name">
									Payment method
								</span>

                                        <span class="price-total">
									<?php echo $payment_method; ?>
								</span>
                                    </div>
                                <?php } ?>

                                <?php if($this->attributes['enable_course_details'] == 'on') {?>
                                    <div class="order-item status-info">
								<span class="course-name">
									Status
								</span>

                                        <span class="price-total">
									<?php
                                    if( $course_type == 'subscribe' ){
                                        if( !empty( $trial_period ) ){
                                            echo $trial_period.' '.strtolower($trial_period_unit_text).' trial '.'('.$currency.$trial_price.')';
                                        }else{
                                            echo $currency.$price;
                                        }

                                    }else{
                                        echo __('Enrolled');

                                        if( $is_expire == 'on' ){
                                            echo '<small> (expires in '.$expire_days.' days)</small>';
                                        }
                                    }
                                    ?>

								</span>
                                    </div>
                                <?php } ?>
                            </div>

                            <div class="order-details-table-footer">
                                <div class="footer-list">
                                    <span class="total-title">Total Payment</span>
                                    <span class="total-price"><?php
                                        if($payment_method == 'free'){
                                            echo $course_details['currency']."0";
                                        }elseif( !empty( $trial_period ) ){
                                            echo $currency.$trial_price;
                                        }else{
                                            echo $course_details['currency'].$price;
                                        }

                                        ?>
									</span>
                                </div>
                            </div>
                        </div>
                        <?php
                    }
                }elseif (isset($data['get']['wpfnl_course_status']) && $data['get']['wpfnl_course_status'] == 'enrolled' ){
                    echo __('Thank you.You are already enrolled');
                }else{
                    echo __('No Order Found');
                }
            }else{
                echo __('Sorry, Please place the element in WPFunnels when learnDash is active ');
            }
        }
        return ob_get_clean();
    }


}
