<?php

namespace WPFunnelsProWebHooks\Events;

use WPFunnelsProWebHooks\Functions\Wpfnl_Pro_Webhook_Functions;
use WPFunnelsPro\Wpfnl_Pro_functions;
use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;
class Wpfnl_Pro_Webhook_LmsOffer {
    
    protected $settings = [];

    protected $cookie_data = [];

    protected $order_id;

    protected $offer_status;

    protected $proudct_id;
    
    protected $step_id;

    public function __construct( $settings, $cookie_data, $offer_status ,$proudct_id , $step_id)
    {
        $this->settings     = $settings;
        $this->offer_status = $offer_status;
        $this->cookie_data  = $cookie_data;
        $this->proudct_id   = $proudct_id;
        $this->step_id   = $step_id;
    }


    

    /**
     * Get event name
     * 
     * @return String
     */
    public function get_event_name(){

        $step_id            = $this->step_id;
        $funnel_id          = get_post_meta($step_id, '_funnel_id', true);
        $step_type          = get_post_meta($step_id, '_step_type', true);
        return ucfirst($step_type).' '.ucfirst($this->offer_status);
    }



    /**
     * Send data to request url through webhook
     */
    public function send_data(){
        $request_url  = Wpfnl_Pro_Webhook_Functions::get_request_url( $this->settings );
        $request_args = $this->prepare_request_args( $this->settings );
        $response = wp_remote_request( $request_url, $request_args );
    }

    
    /**
     * Prepare request arguments
     * 
     * @param Array $settings
     * @return Array $request_args
     */
    public function prepare_request_args( $settings ){
        
        $request_body = $this->prepare_request_body( $this->settings , $this->cookie_data, $this->offer_status );
        $content_type = 'application/json';
        return Wpfnl_Pro_functions::prepare_common_request_args( $this->settings, $content_type, $request_body, $this->get_event_name(), home_url( '/' ) );
    } 


    /**
     * Prepare request body
     * 
     * @param Array $settings
     * @return Array $formatted_body
     */
    private function prepare_request_body( $settings , $cookie_data, $offer_status ){
        $type = $settings['request']['body']['type'];
        $formatted_body = array();
       
        if( $type === 'all' ){
            $formatted_body = $this->prepare_all_body_fields( $cookie_data, $offer_status );
        }else{
            $values = $settings['request']['body']['values'];
            $_body = array();
            $_body = $this->prepare_selected_body_fields( $cookie_data, $offer_status );
            $formatted_body['Event Name']   = $this->get_event_name();
            foreach( $values as $value ){
                if( isset( $_body[$value['value']] )){
                    $formatted_body[$value['key']] = $_body[$value['value']];
                }
            }

        }

        return $formatted_body;
    }


    /**
     * Prepare request body field for type 'all'
     * 
     * @param $order, $order_id, $offer_product
     * @return $formatted_body
     */
    public function prepare_all_body_fields( $cookie_data, $offer_status  ){

        $formatted_body['Date']         = date("Y-m-d h:i");
        $formatted_body['Event Name']   = $this->get_event_name();
        if( $this->proudct_id ){
            $course_details = Wpfnl_lms_learndash_functions::get_course_details_by_id($this->proudct_id);
            $formatted_body['Product ID']   = $this->proudct_id;
            $formatted_body['Product Name'] = $course_details['title'];
            $formatted_body['Product Quantity']  = 1;

            $formatted_body['Total Price']  = $course_details['trial_price'] ? $course_details['currency'].' '.$course_details['trial_price'] : $course_details['currency'].' '.$course_details['price'];
            $formatted_body['Order Status'] = $offer_status === 'accepted' ? 'Enrolled' : 'Not enrolled' ;
            
            if( isset($cookie_data['user_info']) ){
                $formatted_body['Customer Name']    = $cookie_data['user_info']['first_name'].' '.$cookie_data['user_info']['last_name'];
                $formatted_body['Customer Email']    = $cookie_data['user_info']['email'];
            }
            
        }
        
        return $formatted_body;
    }


    /**
     * Prepare request body field for type 'selected'
     * 
     * @param $order, $order_id, $offer_product
     * @return $formatted_body
     */
    public function prepare_selected_body_fields( $cookie_data, $offer_status  ){

        $formatted_body['Date']         = date("Y-m-d h:i");
        if( $this->proudct_id ){
            $course_details = Wpfnl_lms_learndash_functions::get_course_details_by_id($this->proudct_id);
            $formatted_body['product_id']   = $this->proudct_id;
            $formatted_body['product_name'] = $course_details['title'];
            $formatted_body['product_quantity']  = 1;

            $formatted_body['order_total']  = $course_details['trial_price'] ? $course_details['currency'].' '.$course_details['trial_price'] : $course_details['currency'].' '.$course_details['price'];
            $formatted_body['order_status'] = $offer_status === 'accepted' ? 'Enrolled' : 'Not enrolled' ;
            
            if( isset($cookie_data['user_info']) ){
                $formatted_body['customer_name']    = $cookie_data['user_info']['first_name'].' '.$cookie_data['user_info']['last_name'];
                $formatted_body['customer_email']    = $cookie_data['user_info']['email'];
            }
            
        }

        return $formatted_body;
    }

}