<?php

namespace WPFunnelsPro\Lms\Widgets\Elementor;

use Elementor\Controls_Manager;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Widget_Base;
use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;
use WPFunnels\Wpfnl_functions;
use Elementor\Controls_Stack;
if (!defined('ABSPATH')) {
	exit;
} // Exit if accessed directly

/**
 * Funnel sell accept button
 *
 * @since 1.0.0
 */
class LmsCheckout extends Widget_Base
{

	/**
     * Register the widget controls.
     *
     * Adds different input fields to allow the user to change and Wpvrize the widget settings.
     *
     * @since 1.0.0
     *
     * @access protected
     */
    protected function init_controls() {
        if ( version_compare(ELEMENTOR_VERSION, '3.1.0', '>=') ) {
            $this->register_controls();
        } else {
            $this->_register_controls();
        }
    }


	
	/**
	 * Retrieve the widget name.
	 *
	 * @return string Widget name.
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 */
	public function get_name()
	{
		return 'wpfnl-lms-checkout';
	}

	/**
	 * Retrieve the widget title.
	 *
	 * @return string Widget title.
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 */
	public function get_title()
	{
		return __('LMS Checkout', 'wpfnl');
	}

	/**
	 * Retrieve the widget icon.
	 *
	 * @return string Widget icon.
	 * @since 1.0.0
	 *
	 * @access public
	 *order-bump-settings
	 */
	public function get_icon()
	{
		return 'icon-wpfnl checkout-icon';
	}

	/**
	 * Retrieve the list of categories the widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * Note that currently Elementor supports only one category.
	 * When multiple categories passed, Elementor uses the first one.
	 *
	 * @return array Widget categories.
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 */
	public function get_categories()
	{
		return ['wp-funnel'];
	}

	/**
	 * Retrieve the list of scripts the widget depended on.
	 *
	 * Used to set scripts dependencies required to run the widget.
	 *
	 * @return array Widget scripts dependencies.
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 */
	public function get_script_depends()
	{
		return ['wpfnl-lms-checkout-widget'];
	}

	/**
	 * Order Bump Settings
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function order_bump_primary_settings()
	{
		$default = [

		];

		$settings = get_post_meta(get_the_ID(), 'order-bump-settings', true);

		if ($settings) {
			return $settings;
		}

		return $default;
	}


	/**
	 * Register cart controls controls.
	 *
	 * @since x.x.x
	 * @access protected
	 */
	protected function register_controls()
	{
		$this->start_controls_section(
			'section_content',
			[
				'label' => __('LMS Checkout', 'wpfnl'),
				'tab' => Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'order_details_header_title',
			[
				'label' => __('Checkout Header Title', 'wpfnl'),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => 'Order Details',
				'placeholder' => __('Type your text here', 'wpfnl'),
			]
		);

		$this->add_control(
			'course_details_title',
			[
				'label' => __('Course Details Title', 'wpfnl'),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => 'Course Details',
			]
		);
		$this->add_control(
			'course_description',
			[
				'label' => __('Course Description', 'wpfnl'),
				'type' => \Elementor\Controls_Manager::TEXTAREA,
				'default' => __('Briefly describe your course in 2-3 lines.','wpfnl'),
			]
		);

		$this->add_control(
			'plan_details_title',
			[
				'label' => __('Plan Details Title', 'wpfnl'),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => 'Plan Details',
			]
		);

		$this->add_control(
			'course_feature_image',
			[
				'label' => __('Choose Image', 'wpfnl'),
				'type' => \Elementor\Controls_Manager::MEDIA,
				'separator' => 'after'
			]
		);


		//=== Order Bump Start ===//
		$order_bump_enabler = '';
		$ob_settings = $this->order_bump_primary_settings();

		if( count($ob_settings) > 0 ){
            foreach( $ob_settings as $key=>$settings ){
                $index = $key +1;

                $this->add_control(
                    'order_bump_'.$key,
                    [
                        'label' => __('Enable Order bump '.((int)($key) + 1), 'wpfnl'),
                        'type' => \Elementor\Controls_Manager::SELECT,
                        'default' => $settings['isEnabled'] ? 'yes' : 'no',
                        'options' => [
                            'yes' => __('Yes', 'wpfnl'),
                            'no' => __('No', 'wpfnl'),
                        ],
                    ]
                );
                $this->add_control(
                    'order_bump_position_'.$key,
                    [
                        'label' => __( 'Order bump '.  $index .' Position', 'wpfnl'),
                        'type' => \Elementor\Controls_Manager::SELECT,
                        'default' => $ob_settings[$key]['position'],
                        'options' => [
                            'before-lms-order' 				=> __('Before product preview', 'wpfnl'),
                            'after-lms-order' 			=> __('After product preview', 'wpfnl'),

                        ],
                        'condition' => [
                            'order_bump_'.$key => 'yes',
                        ]
                    ]
                );


                $this->add_control(
                    'order_bump_layout_'.$key,
                    [
                        'label' => __('Select Style For Order Bump '. $index, 'wpfnl'),
                        'type' => \Elementor\Controls_Manager::SELECT,
                        'default' => $ob_settings[$key]['selectedStyle'],
                        'options' => [
                            'style1' => __('Style 1', 'wpfnl'),
                            'style2' => __('Style 2', 'wpfnl'),
                        ],
                        'condition' => [
                            'order_bump_'.$key => 'yes',
                            'order_bump_position!' => 'popup',
                        ]
                    ]
                );


                $this->add_control(
                    'order_bump_image_'.$key,
                    [
                        'label' => __('Choose Image for Order Bump '. $index , 'wpfnl'),
                        'type' => \Elementor\Controls_Manager::MEDIA,
                        'default' => [
                            'url' => isset($ob_settings[$key]['productImage']['url']) ? $ob_settings[$key]['productImage']['url'] : '',
                        ],
                        'condition' => [
                            'order_bump_'.$key => 'yes',
                        ]
                    ]
                );


                $this->add_control(
                    'order_bump_checkbox_label_'.$key,
                    [
                        'label' => __('Checkbox Label For Order Bump '.$index, 'wpfnl'),
                        'type' => \Elementor\Controls_Manager::TEXT,
                        'default' => $ob_settings[$key]['checkBoxLabel'],
                        'placeholder' => __('Type your text here', 'wpfnl'),
                        'condition' => [
                            'order_bump_'.$key => 'yes',
                        ]
                    ]
                );

                $this->add_control(
                    'order_bump_product_detail_header_'.$key,
                    [
                        'label' => __('Highlight Text For order Bump '.$index, 'wpfnl'),
                        'type' => \Elementor\Controls_Manager::TEXT,
                        'default' => esc_html__( $settings['highLightText'], 'wpfnl' ),
                        'placeholder' => __('Type your text here', 'wpfnl'),
                        'condition' => [
                            'order_bump_'.$key => 'yes',
                        ]
                    ]
                );

                $this->add_control(
                    'order_bump_product_detail_'.$key,
                    [
                        'label' => __('Product Detail For Order Bump '.$index, 'wpfnl'),
                        'type' => \Elementor\Controls_Manager::TEXTAREA,
                        'rows' => 5,
                        'default' => esc_html__( $settings['productDescriptionText'], 'wpfnl' ),
                        'placeholder' => __('Type your description here', 'wpfnl'),
                        'condition' => [
                            'order_bump' => 'yes',
                        ]
                    ]
                );


            }
        }

		//=== Order Bump End ===//
		$this->end_controls_section();

		$this->register_lms_checkout_style_controls();
	}


	/**
	 * Register cart controls controls.
	 *
	 * @since x.x.x
	 * @access protected
	 */
	protected function _register_controls()
	{
		$this->start_controls_section(
			'section_content',
			[
				'label' => __('LMS Checkout', 'wpfnl'),
				'tab' => Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'order_details_header_title',
			[
				'label' => __('Checkout Header Title', 'wpfnl'),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => 'Order Details',
				'placeholder' => __('Type your text here', 'wpfnl'),
			]
		);

		$this->add_control(
			'course_details_title',
			[
				'label' => __('Course Details Title', 'wpfnl'),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => 'Course Details',
			]
		);
		$this->add_control(
			'course_description',
			[
				'label' => __('Course Description', 'wpfnl'),
				'type' => \Elementor\Controls_Manager::TEXTAREA,
				'default' => __('Briefly describe your course in 2-3 lines.','wpfnl'),
			]
		);

		$this->add_control(
			'plan_details_title',
			[
				'label' => __('Plan Details Title', 'wpfnl'),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => 'Plan Details',
			]
		);

		$this->add_control(
			'course_feature_image',
			[
				'label' => __('Choose Image', 'wpfnl'),
				'type' => \Elementor\Controls_Manager::MEDIA,
				'separator' => 'after'
			]
		);


		//=== Order Bump Start ===//
		$order_bump_enabler = '';
		$ob_settings = $this->order_bump_primary_settings();

		if( count($ob_settings) > 0 ){
            foreach( $ob_settings as $key=>$settings ){
                $index = $key +1;

                $this->add_control(
                    'order_bump_'.$key,
                    [
                        'label' => __('Enable Order bump '.((int)($key) + 1), 'wpfnl'),
                        'type' => \Elementor\Controls_Manager::SELECT,
                        'default' => $settings['isEnabled'] ? 'yes' : 'no',
                        'options' => [
                            'yes' => __('Yes', 'wpfnl'),
                            'no' => __('No', 'wpfnl'),
                        ],
                    ]
                );
                $this->add_control(
                    'order_bump_position_'.$key,
                    [
                        'label' => __( 'Order bump '.  $index .' Position', 'wpfnl'),
                        'type' => \Elementor\Controls_Manager::SELECT,
                        'default' => $ob_settings[$key]['position'],
                        'options' => [
                            'before-lms-order' 				=> __('Before product preview', 'wpfnl'),
                            'after-lms-order' 			=> __('After product preview', 'wpfnl'),

                        ],
                        'condition' => [
                            'order_bump_'.$key => 'yes',
                        ]
                    ]
                );


                $this->add_control(
                    'order_bump_layout_'.$key,
                    [
                        'label' => __('Select Style For Order Bump '. $index, 'wpfnl'),
                        'type' => \Elementor\Controls_Manager::SELECT,
                        'default' => $ob_settings[$key]['selectedStyle'],
                        'options' => [
                            'style1' => __('Style 1', 'wpfnl'),
                            'style2' => __('Style 2', 'wpfnl'),
                        ],
                        'condition' => [
                            'order_bump_'.$key => 'yes',
                            'order_bump_position!' => 'popup',
                        ]
                    ]
                );


                $this->add_control(
                    'order_bump_image_'.$key,
                    [
                        'label' => __('Choose Image for Order Bump '. $index , 'wpfnl'),
                        'type' => \Elementor\Controls_Manager::MEDIA,
                        'default' => [
                            'url' => isset($ob_settings[$key]['productImage']['url']) ? $ob_settings[$key]['productImage']['url'] : '',
                        ],
                        'condition' => [
                            'order_bump_'.$key => 'yes',
                        ]
                    ]
                );


                $this->add_control(
                    'order_bump_checkbox_label_'.$key,
                    [
                        'label' => __('Checkbox Label For Order Bump '.$index, 'wpfnl'),
                        'type' => \Elementor\Controls_Manager::TEXT,
                        'default' => $ob_settings[$key]['checkBoxLabel'],
                        'placeholder' => __('Type your text here', 'wpfnl'),
                        'condition' => [
                            'order_bump_'.$key => 'yes',
                        ]
                    ]
                );

                $this->add_control(
                    'order_bump_product_detail_header_'.$key,
                    [
                        'label' => __('Highlight Text For order Bump '.$index, 'wpfnl'),
                        'type' => \Elementor\Controls_Manager::TEXT,
                        'default' => esc_html__( $settings['highLightText'], 'wpfnl' ),
                        'placeholder' => __('Type your text here', 'wpfnl'),
                        'condition' => [
                            'order_bump_'.$key => 'yes',
                        ]
                    ]
                );

                $this->add_control(
                    'order_bump_product_detail_'.$key,
                    [
                        'label' => __('Product Detail For Order Bump '.$index, 'wpfnl'),
                        'type' => \Elementor\Controls_Manager::TEXTAREA,
                        'rows' => 5,
                        'default' => esc_html__( $settings['productDescriptionText'], 'wpfnl' ),
                        'placeholder' => __('Type your description here', 'wpfnl'),
                        'condition' => [
                            'order_bump' => 'yes',
                        ]
                    ]
                );


            }
        }

		//=== Order Bump End ===//
		$this->end_controls_section();

		$this->register_lms_checkout_style_controls();
	}


	/**
	 * Function to get skin types.
	 *
	 * @since x.x.x
	 * @access protected
	 */
	protected function get_skin_types()
	{
		$skin_options = [];

		$skin_options = [
			'default' => __('Default', 'wpfnl'),
			'style-one' => __('Floating Labels', 'wpfnl'),
		];

		return $skin_options;
	}

	/**
	 * Register LMS Checkout Style Controls.
	 *
	 * @since x.x.x
	 * @access protected
	 */
	protected function register_lms_checkout_style_controls()
	{
		$this->start_controls_section(
			'lms_checkout_style',
			[
				'label' => __('LMS Checkout Style', 'wpfnl'),
				'tab' => Controls_Manager::TAB_STYLE,
			]
		);

		// ----headding style-----
		$this->add_control(
			'lms_checkout_heading_style',
			[
				'label' => __('Header Style', 'wpfnl'),
				'type' => Controls_Manager::HEADING,
				'label_block' => true,
			]
		);

		$this->add_control(
			'lms_checkout_header_bg',
			[
				'label' => __('Background Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-header' => 'background-color: {{VALUE}};'
				],
			]
		);

		$this->add_control(
			'lms_checkout_header_color',
			[
				'label' => __('Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-header h4' => 'color: {{VALUE}};'
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'lms_checkout_header_typography',
				'label' => 'Typography',
				'selector' => '{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-header h4',
			]
		);

		$this->add_control(
			'lms_checkout_header_border_style',
			[
				'label' => __('Border Style', 'wpfnl'),
				'type' => Controls_Manager::SELECT,
				'label_block' => false,
				'default' => '',
				'options' => [
					'' => __('Inherit', 'wpfnl'),
					'solid' => __('Solid', 'wpfnl'),
					'double' => __('Double', 'wpfnl'),
					'dotted' => __('Dotted', 'wpfnl'),
					'dashed' => __('Dashed', 'wpfnl'),
				],
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-header' => 'border-style: {{VALUE}};',
				],
			]
		);
		$this->add_control(
			'lms_checkout_header_border_size',
			[
				'label' => __('Border Width', 'wpfnl'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px'],
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-header' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'lms_checkout_header_border_style!' => '',
				],
			]
		);

		$this->add_control(
			'lms_checkout_header_border_color',
			[
				'label' => __('Border Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-header' => 'border-color: {{VALUE}}!important;',
				],
				'condition' => [
					'lms_checkout_header_border_style!' => '',
				],
			]
		);

		$this->add_responsive_control(
			'lms_checkout_header_radius',
			[
				'label' => __('Radius', 'wpfnl'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px', 'em', '%'],
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-header' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'lms_checkout_header_padding',
			[
				'label' => __('Padding', 'wpfnl'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px', '%', 'em'],
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-header' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}}',
				],
				'separator' => 'after',
			]
		);
		//-----end heading style-----

		// -----checkout body style------
		$this->add_control(
			'checkout_body_style',
			[
				'label' => __('Checkout Body Style', 'wpfnl'),
				'type' => Controls_Manager::HEADING,
				'label_block' => true,
			]
		);

		$this->add_control(
			'lms_checkout_body_bg',
			[
				'label' => __('Body Background Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-body' => 'background-color: {{VALUE}};'
				],
			]
		);

		$this->add_control(
			'lms_checkout_body_title_color',
			[
				'label' => __('Title Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-body .course-title,
					 {{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-body .lms-single-block .lms-block-title,
					 {{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-body .lms-single-block .lms-block-title' => 'color: {{VALUE}};'
				],
			]
		);

		$this->add_control(
			'lms_checkout_body_content_color',
			[
				'label' => __('Body Content Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-body .course-title .no-of-lesson,
					 {{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-body .lms-single-block .course-description,
					 {{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-body .lms-single-block .course-plan' => 'color: {{VALUE}};'
				],
			]
		);

		$this->add_control(
			'lms_checkout_highlighted_text_color',
			[
				'label' => __('Highlighted Text Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-body .lms-single-block .course-plan .primary-color' => 'color: {{VALUE}};'
				],
			]
		);

		$this->add_control(
			'lms_checkout_body_border_style',
			[
				'label' => __('Border Style', 'wpfnl'),
				'type' => Controls_Manager::SELECT,
				'label_block' => false,
				'default' => '',
				'options' => [
					'' => __('Inherit', 'wpfnl'),
					'solid' => __('Solid', 'wpfnl'),
					'double' => __('Double', 'wpfnl'),
					'dotted' => __('Dotted', 'wpfnl'),
					'dashed' => __('Dashed', 'wpfnl'),
				],
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-body' => 'border-style: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'lms_checkout_body_border_size',
			[
				'label' => __('Border Width', 'wpfnl'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px'],
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-body' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'lms_checkout_body_border_style!' => '',
				],
			]
		);

		$this->add_control(
			'lms_checkout_body_border_color',
			[
				'label' => __('Border Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-body' => 'border-color: {{VALUE}}!important;',
				],
				'condition' => [
					'lms_checkout_body_border_style!' => '',
				],
			]
		);

		$this->add_responsive_control(
			'lms_checkout_body_radius',
			[
				'label' => __('Radius', 'wpfnl'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px', 'em', '%'],
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-body' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'lms_checkout_body_padding',
			[
				'label' => __('Padding', 'wpfnl'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px', '%', 'em'],
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-body' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}}',
				],
				'separator' => 'after',
			]
		);
		//-----end checkout body style-----


		// -----checkout footer style------
		$this->add_control(
			'checkout_footer_style',
			[
				'label' => __('Checkout Footer Style', 'wpfnl'),
				'type' => Controls_Manager::HEADING,
				'label_block' => true,
			]
		);

		$this->add_control(
			'lms_checkout_footer_text_color',
			[
				'label' => __('Text Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .footer-title' => 'color: {{VALUE}} !important;',
				],
			]
		);

		$this->add_control(
			'lms_checkout_footer_highlighted_text_color',
			[
				'label' => __('Highlighted Text Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .footer-price' => 'color: {{VALUE}} !important;',
				],
				'separator' => 'after',
			]
		);
		//-----end checkout footer style-----

		// -----checkout button style------
		$this->add_control(
			'checkout_button_style',
			[
				'label' => __('Checkout Button Style', 'wpfnl'),
				'type' => Controls_Manager::HEADING,
				'label_block' => true,
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'lms_checkout_button_typo',
				'label' => 'Typography',
				'selector' => '{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .ld-login-button,
                {{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .btn-join',
			]
		);

		$this->start_controls_tabs('lms_checkout_button_tab');
		$this->start_controls_tab(
			'lms_checkout_button_normal',
			[
				'label' => __('Normal', 'wpfnl'),
			]
		);

		$this->add_control(
			'lms_checkout_button_color',
			[
				'label' => __('Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .ld-login-button,
            		 {{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .btn-join' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'lms_checkout_button_bg',
			[
				'label' => __('Background Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .ld-login-button,
            		 {{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .btn-join' => 'background-color: {{VALUE}}!important;',
				],
			]
		);
		$this->end_controls_tab();

		//----hover style---
		$this->start_controls_tab(
			'lms_checkout_button_hover',
			[
				'label' => __('Hover', 'wpfnl'),
			]
		);

		$this->add_control(
			'lms_checkout_button_color_hvr',
			[
				'label' => __('Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .ld-login-button:hover,
            		 {{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .btn-join:hover' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'lms_checkout_button_bg_hvr',
			[
				'label' => __('Background Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .ld-login-button:hover,
            		 {{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .btn-join:hover' => 'background-color: {{VALUE}}!important;',
				],
			]
		);
		$this->end_controls_tab();
		$this->end_controls_tabs();


		$this->add_control(
			'lms_checkout_button_border_style',
			[
				'label' => __('Border Style', 'wpfnl'),
				'type' => Controls_Manager::SELECT,
				'label_block' => false,
				'default' => '',
				'options' => [
					'' => __('Inherit', 'wpfnl'),
					'solid' => __('Solid', 'wpfnl'),
					'double' => __('Double', 'wpfnl'),
					'dotted' => __('Dotted', 'wpfnl'),
					'dashed' => __('Dashed', 'wpfnl'),
				],
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .ld-login-button,
            		 {{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .btn-join' => 'border-style: {{VALUE}};',
				],
			]
		);
		$this->add_control(
			'lms_checkout_button_border_size',
			[
				'label' => __('Border Width', 'wpfnl'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px'],
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .ld-login-button,
            		 {{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .btn-join' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'lms_checkout_button_border_style!' => '',
				],
			]
		);

		$this->add_control(
			'lms_checkout_button_border_color',
			[
				'label' => __('Border Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .ld-login-button,
            		 {{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .btn-join' => 'border-color: {{VALUE}};',
				],
				'condition' => [
					'lms_checkout_button_border_style!' => '',
				],
			]
		);

		$this->add_responsive_control(
			'lms_checkout_button_radius',
			[
				'label' => __('Radius', 'wpfnl'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px', 'em', '%'],
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .ld-login-button,
                    {{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .btn-join' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'lms_checkout_button_padding',
			[
				'label' => __('Padding', 'wpfnl'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px', '%', 'em'],
				'selectors' => [
					'{{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .ld-login-button,
                    {{WRAPPER}} .wpfnl-lms-checkout .lms-checkout-footer .btn-join' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],

			]
		);

		$this->end_controls_section();
	}


	/**
	 * Render the widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function render()
	{

		$settings       = $this->get_settings_for_display();

		if( wp_doing_ajax() && isset($_REQUEST['action']) && 'elementor_ajax' === $_REQUEST['action'] ) {
			$step_id = absint( $_REQUEST['editor_post_id'] );
		} else {
			$step_id    = get_the_ID();
		}

		$funnel_id = get_post_meta($step_id,'_funnel_id',true);
		$get_learn_dash_setting = Wpfnl_lms_learndash_functions::get_learndash_settings($funnel_id);

		if($get_learn_dash_setting == 'yes'){
			$course = Wpfnl_lms_learndash_functions::get_course_details($step_id);
			if (!empty($course)){

				$course_title 		= isset($course['title']) ? $course['title'] : '';
				$description 		= isset($course['description']) ? $course['description'] : '';
				$no_of_lesson 		= isset($course['no_of_lesson']) ? $course['no_of_lesson'] : '';
				$course_type 		= isset($course['type']) ? $course['type'] : '';
				$billing_cycle 		= isset($course['billing_cycle']) ? $course['billing_cycle'] : '';
				$recurring_time 	= isset($course['recurring_time']) ? $course['recurring_time'] : '';
				$price 				= isset($course['price']) ? $course['price'] : '';
				$trial_price 		= isset($course['trial_price']) ? $course['trial_price'] : '';
				$trial_period 		= isset($course['trial_period']) ? $course['trial_period'] : '';
				$is_expire 			= isset($course['is_expire']) ? $course['is_expire'] : '';
				$expire_days 		= isset($course['expire_days']) ? $course['expire_days'] : '';
				$image 				= isset($course['image']) ? $course['image'] : '';
				$billing_cycle_unit = isset($course['billing_cycle_unit']) ? $course['billing_cycle_unit'] : '';
				$trial_period_unit  = isset($course['trial_period_unit']) ? $course['trial_period_unit'] : '';
				$currency  			= isset($course['currency']) ? $course['currency'] : '';
				$discount_price  	= isset($course['discount_price']) ? $course['discount_price'] : '';


				//----course type------
				if( $course_type == 'subscribe' ) {
					$course_type_text = 'Subscription';

				}else if( $course_type == 'open' || $course_type == 'free' ){
					$course_type_text = 'Free';

				}else if( $course_type == 'paynow' ){
					$course_type_text = 'One-time Payment';

				}else {
					$course_type_text = 'Closed';
				}

				//----billing cycle unit------
				if( $course['billing_cycle_unit'] == 'Y' ) {
					$billing_cycle_unit_text = 'year';

				}else if( $course['billing_cycle_unit'] == 'M' ){
					$billing_cycle_unit_text = 'month';

				}else if( $course['billing_cycle_unit'] == 'W' ){
					$billing_cycle_unit_text = 'week';

				}else {
					$billing_cycle_unit_text = 'day';
				}

				//----trial period unit------
				if( $course['trial_period_unit'] == 'Y' ) {
					$trial_period_unit_text = 'year';

				}else if( $course['trial_period_unit'] == 'M' ){
					$trial_period_unit_text = 'month';

				}else if( $course['trial_period_unit'] == 'W' ){
					$trial_period_unit_text = 'week';

				}else {
					$trial_period_unit_text = 'day';
				}
				do_action( 'wpfunnels/before_elementor_checkout_form', $settings );
				do_action( 'wpfunnels/before_checkout_form', $step_id );


				$order_details_header_title = $settings['order_details_header_title'];
				$course_details_title = $settings['course_details_title'];
				$course_description = $settings['course_description'];
				$plan_details_title = $settings['plan_details_title'];
				$course_feature_image = $settings['course_feature_image']['url'];

				?>


				<div>
					<?php  do_action('wpfunnels/before_lms_order_deatils'); ?>
				</div>

				<div class="wpfnl-reset wpfnl-lms-checkout">
					<div class="lms-checkout-box">

						<?php if( !empty($order_details_header_title) ){ ?>
							<div class="lms-checkout-header">
								<h4><?php echo $order_details_header_title; ?></h4>
							</div>
						<?php } ?>

						<div class="lms-checkout-body">
							<div class="course-image">
								<?php
								if( !empty( $course_feature_image ) ){
									echo '<img src="'.$course_feature_image.'" alt="course image" />';

								}else{
									echo '<img src="'.$image.'" alt="course image" />';
								}
								?>
							</div>

							<div class="course-content">
								<h2 class="course-title">
									<?php echo $course_title ?>
									<span class="no-of-lesson">- <?php echo $no_of_lesson ?> Lessons</span>
								</h2>

								<div class="lms-single-block course-details-block">
									<h4 class="lms-block-title"><?php echo $course_details_title; ?></h4>
									<div class="course-description">
										<?php
											echo $course_description;
										?>
									</div>
								</div>

								<div class="lms-single-block course-plan-block">
									<h4 class="lms-block-title"><?php echo $plan_details_title; ?></h4>
									<div class="course-plan">
										<?php
										if( $course_type == 'subscribe' ){
											if( $discount_price ){
												echo $course_type_text .' - <span class="primary-color"><del>'.$currency.$price.'</del> <ins style="text-decoration:none">'.$currency.$discount_price.'</ins>/'.$billing_cycle_unit_text.'</span>';
											}else{
												echo $course_type_text .' - <span class="primary-color">'.$currency.$price.'/'.$billing_cycle_unit_text.'</span>';
											}

											if( !empty( $trial_period ) ){
												echo '<small> ('.$trial_period.' '.$trial_period_unit_text.' trial period)</small>';
											}

										} else if( $course_type == 'paynow' ){
											
											if( $discount_price ){
												echo $course_type_text .' - <span class="primary-color"><del>'.$currency.$price.'</del>  <ins style="text-decoration:none">'.$currency.$discount_price.'</ins></span>';
											}else{
												echo $course_type_text .' - <span class="primary-color">'.$currency.$price.'</span>';
											}
											

											if( $is_expire == 'on' ){
												echo '<small> (expires in '.$expire_days.' days)</small>';
											}
										}else{
											echo $course_type_text;

											if( $is_expire == 'on' ){
												echo '<small> (Login or register to enroll) (expires in '.$expire_days.' days)</small>';
											}
										}
										?>
									</div>
								</div>
							</div>
						</div>
					</div>
					<div class="lms-checkout-footer">
						<?php
						if( $course_type == 'subscribe' ){
							if( !empty( $trial_period ) ){
								?>

								<p class="footer-title">Course Trial Fees <span class="footer-price"><?php echo $currency.$trial_price; ?></span></p>

							<?php }else{ ?>

								<p class="footer-title">Course Subscription Fees <span class="footer-price"><?php echo $currency.$price; ?></span></p>
								<?php
							}
						}else if( $course_type == 'paynow' ){
							if( $discount_price ){ ?>
								<p class="footer-title">Course Fees <span class="footer-price"><del><?php echo $currency.$price; ?></del> <ins style="text-decoration:none"> <?php echo $currency.$discount_price; ?> </ins></span></p>
							<?php } else{ ?>
								<p class="footer-title">Course Fees <span class="footer-price"><?php echo $currency.$price; ?></span></p>
							<?php }
							?>
						<?php }else{ ?>
							<p class="footer-title">Course Fees <span class="footer-price">Free</span></p>
						<?php } ?>

						<!-- buy now button -->
						<?php
						if ( is_user_logged_in() ) {
							$course_access   = sfwd_lms_has_access( $course[ 'id' ], get_current_user_id() );
							$next_step_url   = Wpfnl_lms_learndash_functions::get_next_step_url( $funnel_id, $step_id ) . '?wpfnl_ld_payment=free&wpfnl_course_status=enrolled';
							$lms_button_text = get_option( 'learndash_settings_custom_labels' );
							$button_text     = !empty( $lms_button_text[ 'button_take_this_course' ] ) ? esc_html( $lms_button_text[ 'button_take_this_course' ] ) : __( 'Take This Course', 'wpfunnels-pro-lms' );

                            if ( $course_access ){
								echo "<a class='btn-default' href='{$next_step_url}' id='wpfnl-lms-access-course'>{$button_text}</a>";
								echo '<span class="wpfnl-lms-access-course-message"></span>';
							}
							else if( 'free' === $course_type ){
								$next_step_url = Wpfnl_lms_learndash_functions::get_next_step_url( $funnel_id, $step_id );
								$user_id       = get_current_user_id();
								echo "<a class='btn-default' href='{$next_step_url}' data-user_id='{$user_id}' data-step_id='{$step_id}' data-course_id='{$course['id']}' id='wpfnl-lms-free-course'>{$button_text}</a>";
								echo '<span class="wpfnl-lms-free-course-message"></span>';
							}
							else {
								echo do_shortcode( "[learndash_payment_buttons course_id='{$course['id']}']" );
							}
						}
						else {
							echo do_shortcode( '[learndash_login]' );
						}
						?>
					</div>
				</div>

				<div>
					<?php  do_action('wpfunnels/after_lms_order_deatils'); ?>
				</div>

				<?php


			}
		}else{
			echo __('Sorry, Please place the element in WPFunnels when learnDash is active ');
		}
	}


	public function render_popup_in_editor()
	{
		if (\Elementor\Plugin::$instance->editor->is_edit_mode()) {

			add_action('woocommerce_before_checkout_form', [$this, 'render_order_bump'], 10);
		}
	}

	/**
	 * Render order bump data with hooks.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	public function render_order_bump()
	{


		$step_id = get_the_ID();
		$order_bump = get_post_meta($step_id, 'order-bump', true);
		$order_bump_settings = get_post_meta($step_id, 'order-bump-settings', true);
		foreach( $order_bump_settings as $key=>$ob_settings ){
			if ( $ob_settings['isEnabled'] && isset($ob_settings['product']) && $ob_settings['product'] != '' ) {
				$this->render_order_bump_template($ob_settings);
			}
		}


	}

	public function render_order_bump_template($settings)
	{
		if (!empty($settings['selectedStyle'])) {

			if ($settings['position'] == 'popup') {
				if (\Elementor\Plugin::$instance->editor->is_edit_mode()) {
					echo '<h5 style="margin-bottom: 15px;"><strong>' . __('To see the pop-up offer in action, please preview or view the page.', 'wpfnl') . '</strong></h5>';
				} else {
					require_once WPFNL_DIR . 'public/modules/checkout/templates-style/order-bump-template-' . $settings['selectedStyle'] . '.php';
				}
			} else {
				require_once WPFNL_DIR . 'public/modules/checkout/templates-style/order-bump-template-' . $settings['selectedStyle'] . '.php';
			}
		}
	}



}
