<?php

namespace WPFunnelsPro\Lms\Widgets\Elementor;

use Elementor\Controls_Manager;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Widget_Base;
use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;
use WPFunnels\Wpfnl_functions;
use Elementor\Controls_Stack;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

/**
 * Funnel sell Reject button
 *
 * @since 1.0.0
 */
class Lms_Order_Details extends Widget_Base
{

	/**
     * Register the widget controls.
     *
     * Adds different input fields to allow the user to change and Wpvrize the widget settings.
     *
     * @since 1.0.0
     *
     * @access protected
     */
    protected function init_controls() {
        if ( version_compare(ELEMENTOR_VERSION, '3.1.0', '>=') ) {
            $this->register_controls();
        } else {
            $this->_register_controls();
        }
    }

	/**
	 * Retrieve the widget name.
	 *
	 * @return string Widget name.
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 */
	public function get_name()
	{
		return 'wpfnl-lms-order-detail';
	}

	/**
	 * Retrieve the widget title.
	 *
	 * @return string Widget title.
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 */
	public function get_title()
	{
		return __('LMS Order Detail', 'wpfnl');
	}

	/**
	 * Retrieve the widget icon.
	 *
	 * @return string Widget icon.
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 */
	public function get_icon()
	{
		return 'icon-wpfnl order-details';
	}

	/**
	 * Retrieve the list of categories the widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * Note that currently Elementor supports only one category.
	 * When multiple categories passed, Elementor uses the first one.
	 *
	 * @return array Widget categories.
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 */
	public function get_categories()
	{
		return ['wp-funnel'];
	}

	/**
	 * Retrieve the list of scripts the widget depended on.
	 *
	 * Used to set scripts dependencies required to run the widget.
	 *
	 * @return array Widget scripts dependencies.
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 */
	public function get_script_depends()
	{
		return ['funnel-lms-order-detail-widget'];
	}

	/**
	 * Register the widget controls.
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function register_controls()
	{
		$this->lms_order_details_controls();
		$this->register_lms_order_details_style_controls();

	}

	/**
	 * Register the widget controls.
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function _register_controls()
	{
		$this->lms_order_details_controls();
		$this->register_lms_order_details_style_controls();

	}

	/**
	 * LMS order details controls.
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function lms_order_details_controls(){
		$this->start_controls_section(
			'section_content',
			[
				'label' => __('Funnel Lms Order Details', 'wpfnl'),
			]
		);

		$this->add_control(
			'enable_course_review',
			[
				'label' => __('Enable Course Overview', 'wpfnl'),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'default' => 'on',
				'label_on' => __( 'On', 'wpfnl' ),
				'label_off' => __( 'Off', 'wpfnl' ),
				'return_value' => 'on',
			]
		);

		$this->add_control(
			'enable_payment_method',
			[
				'label' => __('Enable Payment Method', 'wpfnl'),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'default' => 'on',
				'label_on' => __( 'On', 'wpfnl' ),
				'label_off' => __( 'Off', 'wpfnl' ),
				'return_value' => 'on',
			]
		);

		$this->add_control(
			'enable_course_details',
			[
				'label' => __('Enable Course Status', 'wpfnl'),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'default' => 'on',
				'label_on' => __( 'On', 'wpfnl' ),
				'label_off' => __( 'Off', 'wpfnl' ),
				'return_value' => 'on',
			]
		);

		$this->end_controls_section();
	}

	/**
	 * Register LMS Order Details Style Controls.
	 *
	 * @since x.x.x
	 * @access protected
	 */
	protected function register_lms_order_details_style_controls()
	{
		$this->start_controls_section(
			'lms_order_details_style',
			[
				'label' => __('LMS Order Details Style', 'wpfnl'),
				'tab' => Controls_Manager::TAB_STYLE,
			]
		);

		// -----order details global style------
		$this->add_control(
			'lms_order_details_global_style',
			[
				'label' => __('Global Style', 'wpfnl'),
				'type' => Controls_Manager::HEADING,
				'label_block' => true,
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'lms_order_details_global_text_typography',
				'label' => 'Typography',
				'selector' => '{{WRAPPER}} .lms-order-details .order-details-table-footer .footer-list span, {{WRAPPER}} .lms-order-details .order-details-title, {{WRAPPER}} .lms-order-details .order-details-title span',
			]
		);

		$this->add_control(
			'lms_order_details_global_text_color',
			[
				'label' => __('Global Text Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table-footer .footer-list span,
					 {{WRAPPER}} .lms-order-details .order-details-title,
					 {{WRAPPER}} .lms-order-details .order-details-title span' => 'color: {{VALUE}} !important;',
				],
				'separator' => 'after',
			]
		);
		//-----end order details global style-----

		// ----headding style-----
		$this->add_control(
			'lms_order_details_heading_style',
			[
				'label' => __('Header Style', 'wpfnl'),
				'type' => Controls_Manager::HEADING,
				'label_block' => true,
			]
		);

		$this->add_control(
			'lms_order_details_header_bg',
			[
				'label' => __('Background Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table .order-tbl-header' => 'background-color: {{VALUE}};'
				],
			]
		);

		$this->add_control(
			'lms_order_details_header_color',
			[
				'label' => __('Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table .order-tbl-header > span' => 'color: {{VALUE}}!important;'
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'lms_order_details_header_typography',
				'label' => 'Typography',
				'selector' => '{{WRAPPER}} .lms-order-details .order-details-table .order-tbl-header > span',
			]
		);

		$this->add_control(
			'lms_order_details_header_border_style',
			[
				'label' => __('Border Style', 'wpfnl'),
				'type' => Controls_Manager::SELECT,
				'label_block' => false,
				'default' => '',
				'options' => [
					'' => __('Inherit', 'wpfnl'),
					'solid' => __('Solid', 'wpfnl'),
					'double' => __('Double', 'wpfnl'),
					'dotted' => __('Dotted', 'wpfnl'),
					'dashed' => __('Dashed', 'wpfnl'),
				],
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table .order-tbl-header' => 'border-style: {{VALUE}}!important;',
				],
			]
		);

		$this->add_control(
			'lms_order_details_header_border_size',
			[
				'label' => __('Border Width', 'wpfnl'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px'],
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table .order-tbl-header' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}}!important;',
				],
				'condition' => [
					'lms_order_details_header_border_style!' => '',
				],
			]
		);

		$this->add_control(
			'lms_order_details_header_border_color',
			[
				'label' => __('Border Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table .order-tbl-header' => 'border-color: {{VALUE}}!important;',
				],
				'condition' => [
					'lms_order_details_header_border_style!' => '',
				],
			]
		);

		$this->add_responsive_control(
			'lms_order_details_header_radius',
			[
				'label' => __('Radius', 'wpfnl'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px', 'em', '%'],
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table .order-tbl-header' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}}!important;',
				],
			]
		);

		$this->add_responsive_control(
			'lms_order_details_header_padding',
			[
				'label' => __('Padding', 'wpfnl'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px', '%', 'em'],
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table .order-tbl-header' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}}!important;',
				],
				'separator' => 'after',
			]
		);
		//-----end heading style-----

		// -----Order details body style------
		$this->add_control(
			'lms_order_details_body_style',
			[
				'label' => __('Body Style', 'wpfnl'),
				'type' => Controls_Manager::HEADING,
				'label_block' => true,
			]
		);

		$this->add_control(
			'lms_order_details_body_bg',
			[
				'label' => __('Body Background Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table' => 'background-color: {{VALUE}};'
				],
			]
		);

		$this->add_control(
			'lms_order_details_body_title_color',
			[
				'label' => __('Body Content Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table .order-item > span' => 'color: {{VALUE}};'
				],
			]
		);

		$this->add_control(
			'lms_order_details_highlighted_text_color',
			[
				'label' => __('Highlighted Text & Link Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table .course-info > span a,
					 {{WRAPPER}} .lms-order-details .order-details-table .order-item a' => 'color: {{VALUE}};'
				],
			]
		);

		$this->add_control(
			'lms_order_details_body_border_style',
			[
				'label' => __('Border Style', 'wpfnl'),
				'type' => Controls_Manager::SELECT,
				'label_block' => false,
				'default' => '',
				'options' => [
					'' => __('Inherit', 'wpfnl'),
					'solid' => __('Solid', 'wpfnl'),
					'double' => __('Double', 'wpfnl'),
					'dotted' => __('Dotted', 'wpfnl'),
					'dashed' => __('Dashed', 'wpfnl'),
				],
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table' => 'border-style: {{VALUE}};',
					'{{WRAPPER}} .lms-order-details .order-details-table .order-item' => 'border-bottom-style: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'lms_order_details_body_border_size',
			[
				'label' => __('Border Width', 'wpfnl'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px'],
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} .lms-order-details .order-details-table .order-item' => 'border-bottom-width: {{BOTTOM}}{{UNIT}};',
					'{{WRAPPER}} .lms-order-details .order-details-table .order-item:last-child' => 'border-bottom: none;',
				],
				'condition' => [
					'lms_order_details_body_border_style!' => '',
				],
			]
		);

		$this->add_control(
			'lms_order_details_body_border_color',
			[
				'label' => __('Border Color', 'wpfnl'),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table' => 'border-color: {{VALUE}};',
					'{{WRAPPER}} .lms-order-details .order-details-table .order-item' => 'border-bottom-color: {{VALUE}};',
				],
				'condition' => [
					'lms_order_details_body_border_style!' => '',
				],
			]
		);

		$this->add_responsive_control(
			'lms_order_details_body_radius',
			[
				'label' => __('Radius', 'wpfnl'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px', 'em', '%'],
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'lms_order_details_body_padding',
			[
				'label' => __('Padding', 'wpfnl'),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => ['px', '%', 'em'],
				'selectors' => [
					'{{WRAPPER}} .lms-order-details .order-details-table .order-item' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}}',
				],
				'separator' => 'after',
			]
		);
		//-----end checkout body style-----

		$this->end_controls_section();
	}

	/**
	 * Render the widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function render()
	{
		$settings = $this->get_settings();

		if( wp_doing_ajax() && isset($_REQUEST['action']) && 'elementor_ajax' === $_REQUEST['action'] ) {
			$step_id = absint( $_REQUEST['editor_post_id'] );
		} else {
			$step_id    = get_the_ID();
		}

		$funnel_id = get_post_meta($step_id,'_funnel_id',true);
		$get_learn_dash_setting = Wpfnl_lms_learndash_functions::get_learndash_settings($funnel_id);
		if($get_learn_dash_setting == 'yes'){
			$order_details = Wpfnl_lms_learndash_functions::get_lms_order_details( get_current_user_id() ,$funnel_id );
		
			if (!empty($order_details)){
				foreach($order_details as $details){
					$course_details =  $details['course_details'];
					$payment_method =  $details['payment_method'];

					$course_type 		= $course_details['type'];
					$billing_cycle 		= $course_details['billing_cycle'];
					$recurring_time 	= $course_details['recurring_time'];
					$price 				= $course_details['price'];
					$trial_price 		= $course_details['trial_price'];
					$trial_period 		= $course_details['trial_period'];
					$is_expire 			= $course_details['is_expire'];
					$expire_days 		= $course_details['expire_days'];
					$currency  			= $course_details['currency'];
					$price  			= isset($details['amount']) ? $details['amount'] : $price;

					//----course type------
					if( $course_type == 'subscribe' ) {
						$course_type_text = 'Subscription';

					}else if( $course_type == 'open' || $course_type == 'free' ){
						$course_type_text = 'Free';

					}else if( $course_type == 'paynow' ){
						$course_type_text = 'One-time Payment';

					}else {
						$course_type_text = 'Closed';
					}

					//----billing cycle unit------
					if( $course_details['billing_cycle_unit'] == 'Y' ) {
						$billing_cycle_unit_text = 'year';

					}else if( $course_details['billing_cycle_unit'] == 'M' ){
						$billing_cycle_unit_text = 'month';

					}else if( $course_details['billing_cycle_unit'] == 'W' ){
						$billing_cycle_unit_text = 'week';

					}else {
						$billing_cycle_unit_text = 'day';
					}

					//----trial period unit------
					if( $course_details['trial_period_unit'] == 'Y' ) {
						$trial_period_unit_text = 'year';

					}else if( $course_details['trial_period_unit'] == 'M' ){
						$trial_period_unit_text = 'month';

					}else if( $course_details['trial_period_unit'] == 'W' ){
						$trial_period_unit_text = 'week';

					}else {
						$trial_period_unit_text = 'day';
					}

					?>
					<div class="lms-order-details">
						<!-- <h2 class="order-details-title"><span>Thank you.</span> You have been enrolled to your courses.</h2> -->

						<div class="order-details-table">
							<div class="order-item order-tbl-header">
								<?php
									if( $details['step_type'] == 'upsell' || $details['step_type'] == 'downsell' ){
										?>
											<span class="course-name">Course Offer</span>
										<?php
									}else{
										?>
										<span class="course-name">Course</span>
									<?php
									}
								?>


								<span class="price-total">Total</span>
							</div>
							<?php if($settings['enable_course_review'] == 'on') {?>
								<div class="order-item course-info">
								<span class="course-name">
									<span><?php echo $course_details['title']; ?></span>
									<a href="<?php echo get_post_permalink($course_details['id']) ?>">View Course</a>
								</span>

								<span class="price-total">

									<?php
									if( $course_type == 'subscribe' ){

										echo  $currency.$price.'/'.$billing_cycle_unit_text;

									} else if( $course_type == 'paynow' ){
										echo $currency.$price;
									}else{
										echo $course_type_text;
									}
									?>


									<?php //echo $course_details['currency'].$course_details['price']; ?>
								</span>
							</div>
							<?php } ?>

							<?php if($settings['enable_payment_method'] == 'on') {?>
								<div class="order-item">
								<span class="course-name">
									Payment method
								</span>

								<span class="price-total">
									<?php echo $payment_method; ?>
								</span>
							</div>
							<?php } ?>

							<?php if($settings['enable_course_details'] == 'on') {?>
								<div class="order-item">
								<span class="course-name">
									Status
								</span>

								<span class="price-total">
									<?php
									if( $course_type == 'subscribe' ){
										if( !empty( $trial_period ) ){
											echo $trial_period.' '.strtolower($trial_period_unit_text).' trial '.'('.$currency.$trial_price.')';
										}else{
											echo $currency.$price;
										}

									}else{
										echo __('Enrolled');

										if( $is_expire == 'on' ){
											echo '<small> (expires in '.$expire_days.' days)</small>';
										}
									}
									?>

								</span>
							</div>
							<?php } ?>
						</div>

						<div class="order-details-table-footer">
							<div class="footer-list">
								<span class="total-title">Total Payment</span>
								<span class="total-price"><?php
									if($payment_method == 'free'){
										echo $course_details['currency']."0";
									}elseif( !empty( $trial_period ) ){
											echo $currency.$trial_price;
									}else{
										echo $course_details['currency'].$price;
									}

									?>
								</span>
							</div>
						</div>
					</div>
					<?php
				}
			}elseif (isset($_GET['wpfnl_course_status']) && $_GET['wpfnl_course_status'] == 'enrolled' ){
				echo __('Thank you.You are already enrolled');
			}
			else {
				echo __('No Order Found');
			}

		}
	}


}
