<?php

namespace WPFunnelsPro\Lms\Widgets\Elementor;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Icons_Manager;
use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;
use WPFunnelsPro\OfferProduct\Wpfnl_Offer_Product;
use WPFunnelsPro\Wpfnl_Pro_functions;
use Elementor\Core\Schemes\Color as Scheme_Color;
use Elementor\Controls_Stack;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

/**
 * Funnel sell accept button
 *
 * @since 1.0.0
 */
class LmsPayButton extends Widget_Base
{


    /**
     * Register the widget controls.
     *
     * Adds different input fields to allow the user to change and Wpvrize the widget settings.
     *
     * @since 1.0.0
     *
     * @access protected
     */
    protected function init_controls() {
        if ( version_compare(ELEMENTOR_VERSION, '3.1.0', '>=') ) {
            $this->register_controls();
        } else {
            $this->_register_controls();
        }
    }



    /**
     * Retrieve the widget name.
     *
     * @return string Widget name.
     * @since 1.0.0
     *
     * @access public
     *
     */
    public function get_name()
    {
        return 'wpfnl-pay-button';
    }

    /**
     * Retrieve the widget title.
     *
     * @return string Widget title.
     * @since 1.0.0
     *
     * @access public
     *
     */
    public function get_title()
    {
        return __('LMS Pay Button', 'wpfnl');
    }

    /**
     * Retrieve the widget icon.
     *
     * @return string Widget icon.
     * @since 1.0.0
     *
     * @access public
     *
     */
    public function get_icon()
    {
        return 'icon-wpfnl sell-accept';
    }

    /**
     * Retrieve the list of categories the widget belongs to.
     *
     * Used to determine where to display the widget in the editor.
     *
     * Note that currently Elementor supports only one category.
     * When multiple categories passed, Elementor uses the first one.
     *
     * @return array Widget categories.
     * @since 1.0.0
     *
     * @access public
     *
     */
    public function get_categories()
    {
        return ['wp-funnel'];
    }

    /**
     * Retrieve the list of scripts the widget depended on.
     *
     * Used to set scripts dependencies required to run the widget.
     *
     * @return array Widget scripts dependencies.
     * @since 1.0.0
     *
     * @access public
     *
     */
    public function get_script_depends()
    {
        return ['upsell-downsell-lms-widget'];
    }

    /**
     * Get button sizes.
     *
     * Retrieve an array of button sizes for the button widget.
     *
     * @return array An array containing button sizes.
     * @since 1.0.0
     * @access public
     * @static
     *
     */
    public static function get_button_sizes()
    {
        return [
            'xs' => __('Extra Small', 'wpfnl-pro'),
            'sm' => __('Small', 'wpfnl-pro'),
            'md' => __('Medium', 'wpfnl-pro'),
            'lg' => __('Large', 'wpfnl-pro'),
            'xl' => __('Extra Large', 'wpfnl-pro'),
        ];
    }


    /**
     * Register the widget controls.
     * @since 1.0.0
     *
     * @access protected
     */
    protected function register_controls()
    {
        $this->lms_pay_button_controls();
    }

    /**
     * Register the widget controls.
     * @since 1.0.0
     *
     * @access protected
     */
    protected function _register_controls()
    {
        $this->lms_pay_button_controls();
    }

    /**
     * Pay button controls.
     * @since 1.0.0
     *
     * @access protected
     */
    protected function lms_pay_button_controls(){
        $this->start_controls_section(
            'section_button',
            array(
                'label' => __('Upsell/Downsell', 'wpfnl'),
            )
        );

        $this->add_control(
            'offer_button_type',
            [
                'label' => __('Select button type', 'wpfnl'),
                'type' => \Elementor\Controls_Manager::SELECT,
                'default' => 'upsell',
                'options' => [
                    'upsell' => __('Upsell', 'wpfnl'),
                    'downsell' => __('Downsell', 'wpfnl'),
                ],
            ]
        );

        $this->add_control(
            'offer_type',
            [
                'label' => __('Select button action', 'wpfnl'),
                'type' => \Elementor\Controls_Manager::SELECT,
                'default' => 'accept',
                'options' => array(
                    'accept' => __('Accept', 'wpfnl'),
                    'reject' => __('Reject', 'wpfnl'),
                )
            ]
        );

        $this->add_control(
            'text',
            [
                'label' => __('Text', 'wpfnl-pro'),
                'type' => Controls_Manager::TEXT,
                'default' => __('I will pass', 'wpfnl-pro'),
                'placeholder' => __('I will pass', 'wpfnl-pro'),
                'condition' => [
                    'offer_type' => 'reject',
                ],
                'separator' => 'before',
            ]
        );

        $this->add_responsive_control(
            'align',
            [
                'label' => __('Alignment', 'wpfnl-pro'),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => __('Left', 'wpfnl-pro'),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => __('Center', 'wpfnl-pro'),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => __('Right', 'wpfnl-pro'),
                        'icon' => 'fa fa-align-right',
                    ],
                    'justify' => [
                        'title' => __('Justified', 'wpfnl-pro'),
                        'icon' => 'fa fa-align-justify',
                    ],
                ],
                'prefix_class' => 'elementor%s-align-',
                'default' => '',
            ]
        );

        $this->add_control(
            'view',
            [
                'label' => __('View', 'wpfnl-pro'),
                'type' => Controls_Manager::HIDDEN,
                'default' => 'traditional',
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_style',
            [
                'label' => __('Button', 'wpfnl-pro'),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'upsell_downsell_button_typography',
                'label' => 'Typography',
                'selector' => '{{WRAPPER}} a.elementor-button, {{WRAPPER}} .btn-join',
            ]
        );

        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'upsell_downsell_button_text_shadow',
                'selector' => '{{WRAPPER}} a.elementor-button, {{WRAPPER}} .btn-join',
            ]
        );

        $this->start_controls_tabs('tabs_button_style');

        $this->start_controls_tab(
            'tab_button_normal',
            [
                'label' => __('Normal', 'wpfnl-pro'),
            ]
        );

        $this->add_control(
            'button_text_color',
            [
                'label' => __('Text Color', 'wpfnl-pro'),
                'type' => Controls_Manager::COLOR,
                'default' => '',
                'selectors' => [
                    '{{WRAPPER}} a.elementor-button,
                    {{WRAPPER}} .btn-join,
                    {{WRAPPER}} .elementor-button' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'background_color',
            [
                'label' => __('Background Color', 'wpfnl-pro'),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Scheme_Color::get_type(),
                    'value' => Scheme_Color::COLOR_4,
                ],
                'default' => '#61CE70',
                'selectors' => [
                    '{{WRAPPER}} a.elementor-button, 
                    {{WRAPPER}} .btn-join,
                    {{WRAPPER}} .elementor-button' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab(
            'tab_button_hover',
            [
                'label' => __('Hover', 'wpfnl-pro'),
            ]
        );

        $this->add_control(
            'hover_color',
            [
                'label' => __('Text Color', 'wpfnl-pro'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} a.elementor-button:hover, 
                    {{WRAPPER}} .btn-join:hover,
                    {{WRAPPER}} .elementor-button:hover' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'button_background_hover_color',
            [
                'label' => __('Background Color', 'wpfnl-pro'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} a.elementor-button:hover, 
                    {{WRAPPER}} .btn-join:hover,
                    {{WRAPPER}} .elementor-button:hover' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'button_hover_border_color',
            [
                'label' => __('Border Color', 'wpfnl-pro'),
                'type' => Controls_Manager::COLOR,
                'condition' => [
                    'border_border!' => '',
                ],
                'selectors' => [
                    '{{WRAPPER}} a.elementor-button:hover, 
                    {{WRAPPER}} .btn-join:hover,
                    {{WRAPPER}} .elementor-button:hover' => 'border-color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'border',
                'label' => __('Border', 'wpfnl-pro'),
                'placeholder' => '1px',
                'default' => '1px',
                'selector' => '{{WRAPPER}} .elementor-button, {{WRAPPER}} .btn-join',
                'separator' => 'before',
            ]
        );

        $this->add_control(
            'border_radius',
            [
                'label' => __('Border Radius', 'wpfnl-pro'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors' => [
                    '{{WRAPPER}} a.elementor-button, 
                    {{WRAPPER}} .btn-join,
                    {{WRAPPER}} .elementor-button' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'button_box_shadow',
                'selector' => '{{WRAPPER}} .elementor-button, {{WRAPPER}} .btn-join',
            ]
        );

        $this->add_responsive_control(
            'upsell_downsell_padding',
            [
                'label' => __('Padding', 'wpfnl-pro'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', 'em', '%'],
                'selectors' => [
                    '{{WRAPPER}} a.elementor-button, 
                    {{WRAPPER}} .btn-join,
                    {{WRAPPER}} .elementor-button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'separator' => 'before',
            ]
        );

        $this->end_controls_section();
    }

    public function get_prev_next_link_options()
    {
        $associate_funnel_id = get_post_meta(get_the_ID(), '_funnel_id', true);
        $steps_array = [
            'upsell' => 'Upsell',
            'downsell' => 'Downsell',
            'thankyou' => 'Thankyou'
        ];
        $option_group = [];
        foreach ($steps_array as $key => $value) {
            $args = [
                'posts_per_page' => -1,
                'orderby' => 'date',
                'order' => 'DESC',
                'post_type' => WPFNL_STEPS_POST_TYPE,
                'post_status' => 'publish',
                'post__not_in' => [$this->get_id()],
                'meta_query' => [
                    'relation' => 'AND',
                    [
                        'key' => '_step_type',
                        'value' => $key,
                        'compare' => '=',
                    ],
                    [
                        'key' => '_funnel_id',
                        'value' => $associate_funnel_id,
                        'compare' => '=',
                    ],
                ],
            ];
            $query = new \WP_Query($args);
            $steps = $query->posts;
            if ($steps) {
                foreach ($steps as $s) {
                    $option_group[$key][] = [
                        'id' => $s->ID,
                        'title' => $s->post_title,
                    ];
                }
            }
        }
        return $option_group;
    }

    /**
     * Get all WC products
     * @since 1.0.0
     *
     * @access protected
     */
    protected function get_products_array()
    {
        $products = array();
        if (in_array('woocommerce/woocommerce.php', WPFNL_ACTIVE_PLUGINS)) {
            $ids = wc_get_products(array('return' => 'ids', 'limit' => -1));
            foreach ($ids as $id) {
                $title = get_the_title($id);
                $products[$id] = $title;
            }
        }
        return $products;
    }

    /**
     * Get all funnel steps
     * @since 1.0.0
     *
     * @access protected
     */
    protected function get_steps_array($type = 'upsell')
    {
        $options = $this->get_prev_next_link_options();
        $response = array();
        if (isset($options[$type])) {
            $prime_data = $options[$type];
            foreach ($prime_data as $data) {
                $response[$data['id']] = $data['title'];
            }
        }

        return $response;
    }

    /**
     * Render the widget output on the frontend.
     *
     * Written in PHP and used to generate the final HTML.
     *
     * @since 1.0.0
     *
     * @access protected
     */
    protected function render()
    {
        if( wp_doing_ajax() && isset($_REQUEST['action']) && 'elementor_ajax' === $_REQUEST['action'] ) {
            $step_id = absint( $_REQUEST['editor_post_id'] );
        } else {
            $step_id    = get_the_ID();
        }

        $step_type = get_post_meta($step_id,'_step_type',true);
        $funnel_id = get_post_meta($step_id,'_funnel_id',true);
        $lms_button_text = get_option( 'learndash_settings_custom_labels' );

        $settings = $this->get_settings();
        $this->add_render_attribute('wrapper', 'class', 'elementor-button-wrapper');

        $this->add_render_attribute('button', 'class', 'elementor-button');
        $this->add_render_attribute(
            'button',
            array(
                'id' => ['wpfunnels_'.$settings['offer_button_type'].'_'.$settings['offer_type']],
                'class' => [ 'wpfunnels-elementor-widget', 'wpfunnels_offer_button' ],
            )
        );

        ?>
         <?php
         
        if ($settings['offer_type'] == 'accept'){
            $button_text = !empty($lms_button_text['button_take_this_course']) ? $lms_button_text['button_take_this_course'] : "Take this course";

            $lms_product_id = get_post_meta($step_id,'_wpfnl_upsell_products',true);

            if ($step_type == 'downsell'){
                $lms_product_id = get_post_meta($step_id,'_wpfnl_downsell_products',true);
            }
            
            $course_access = sfwd_lms_has_access( $lms_product_id[0]['id'], get_current_user_id() );
            $next_step_url = Wpfnl_lms_learndash_functions::get_next_step_url($funnel_id,$step_id).'?wpfnl_ld_payment=free';
            if ($course_access ){
                echo '<a class="btn-default btn-join elementor-button" href="'.$next_step_url.'" id="wpfnl-lms-access-course">'.$button_text.'</a>';
                echo '<span class="wpfnl-lms-access-course-message"></span>';
            }else{
                if( \Elementor\Plugin::$instance->editor->is_edit_mode()) {
                    echo '<div class="learndash-block-inner"><a class="btn-join elementor-button" href="#" id="btn-join"> '.$button_text.'</a></div>';
                } else {
                    $course = isset($lms_product_id[0]['id']) ? Wpfnl_lms_learndash_functions::get_course_details_by_id($lms_product_id[0]['id']) : [];
                    if( !empty($course) ){
                        if( isset($course['type'])  && $course['type'] == 'free' ){
                            $next_step_url = Wpfnl_lms_learndash_functions::get_next_step_url($funnel_id,$step_id);
                            echo '<a class="btn-default" href="'.$next_step_url.'" data-user_id="'.get_current_user_id().'" data-step_id="'.$step_id.'" data-course_id="'.$course['id'].'" id="wpfnl-lms-free-course">'.$button_text.'</a>';
                            echo '<span class="wpfnl-lms-free-course-message"></span>';
                        }
                        else{
                            echo do_shortcode( "[learndash_payment_buttons course_id='{$course['id']}']" );
                        }
                    }
                    
                }
            }

        }else{
            ?>
            <div <?php echo $this->get_render_attribute_string('wrapper'); ?>>
                <a
                        href="#"
                    <?php echo $this->get_render_attribute_string('button'); ?>
                        id="wpfunnels_<?php echo $settings['offer_button_type']; ?>_<?php echo $settings['offer_type']; ?>"
                        data-offertype="<?php echo $settings['offer_button_type']; ?>" >
                    <?php $this->render_text(); ?>
                </a>
            </div>
            <?php
        }

    }


    /**
     * Render button text.
     *
     * Render button widget text.
     *
     * @since 1.5.0
     * @access protected
     */
    protected function render_text()
    {
        $settings = $this->get_settings();

        $this->add_render_attribute([
            'content-wrapper' => [
                'class' => 'elementor-button-content-wrapper',
            ],
            'text' => [
                'class' => 'elementor-button-text',
            ],
        ]);


        $this->add_render_attribute('content-wrapper', 'class', 'elementor-button-content-wrapper');
        $this->add_render_attribute('text', 'class', 'elementor-button-text');
        $this->add_inline_editing_attributes('text', 'none');
        ?>
        <span <?php echo $this->get_render_attribute_string('content-wrapper'); ?>>
            <span <?php echo $this->get_render_attribute_string('text'); ?>><?php echo $settings['text']; ?></span>
        </span>
        <?php
    }

}