(function( $ ) {
	'use strict';

	/**
	 * All the code for your public-facing JavaScript source
	 * should reside in this file.
	 *
	 * Note: It has been assumed you will write jQuery code here, so the
	 * $ function reference has been prepared for usage within the scope
	 * of this function.
	 *
	 * This enables you to define handlers, for when the DOM is ready:
	 *
	 * $(function() {
	 *
	 * });
	 *
	 * When the window is loaded:
	 *
	 * $( window ).load(function() {
	 *
	 * });
	 *
	 * ...and/or other possibilities.
	 *
	 * Ideally, it is not considered the best practise to attach more than a
	 * single DOM-ready or window-load handler for a particular page.
	 * Although scripts in the WordPress core, Plugins and Themes may be
	 * practising this, we should strive to set a better example in our own work.
	 */

	$( document ).on( 'click', '#wpfnl-lms-free-course', function (e) {
		e.preventDefault();
		$(this).html('Enrolling... <span class="wpfnl-loader"></span>');
		const ajaxurl = window.wpfnl_obj.ajaxurl,
			step_id = $(this).data('step_id'),
			course_id = $(this).data('course_id'),
			user_id = $(this).data('user_id'),
			data = {
				action: 'wpfnl_learndash_free_course',
				step_id: step_id,
				course_id: course_id,
				user_id: user_id,
				data: $(this).serialize(),
			};

		$.ajax({
			type: "POST",
			url: ajaxurl,
			data: data,
			dataType: 'json',
			success: function (response) {
				if (response.success) {
					setTimeout(function () {
						window.location = response.next_step;
					}, 1000);
				}
			}
		});
	});
})( jQuery );
