<?php

namespace WPFunnelsPro\Lms\Frontend\Modules\Webhook;

use WPFunnelsProWebHooks\Functions\Wpfnl_Pro_Webhook_Functions;
use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;
use WPFunnelsProWebHooks\Events;
use WPFunnels\Wpfnl_functions;
use WPFunnels\Data_Store\Wpfnl_Steps_Store_Data;
class Wpfnl_Pro_Lms_Webhook_Mapping {


    public $funnel_id;
    public $settings;
    public function __construct()
    {
        add_action( 'wpfunnels/lms_course_enroll', array($this, 'lms_course_enroll'), 10, 2 );
    }


    /**
     * Trigger when lms course enroll through funnel
     * 
     * @param Array $cookie_data
     */
    public function lms_course_enroll( $cookie_data, $step_id ){

        if( $step_id ){
            $step_type = get_post_meta( $step_id, '_step_type', true );
            if( 'checkout' === $step_type ){
                
                $this->orderbump_triggered( $cookie_data , $step_id, $step_type );
            }elseif( 'upsell' === $step_type || 'downsell' === $step_type ){
                if( is_plugin_active( 'wpfunnels-pro/wpfnl-pro.php' )){
                    $this->offer_triggered( $cookie_data , $step_id, $step_type );
                }
                
            }
        }
        
    }


    /**
     * Allow to send data when orderbump triggered
     */
    private function orderbump_triggered( $cookie_data , $step_id, $step_type ){
        if( $step_id && isset($cookie_data['funnel_id']) ){
            $is_ob_settings = get_post_meta( $step_id , 'order-bump-settings' , true );
            $settings   =  Wpfnl_Pro_Webhook_Functions::get_webhook_settings( $cookie_data['funnel_id'] );
            
            if(!empty($is_ob_settings) && !empty($settings) ){
               
                foreach( $settings as $key=>$setting ){
                    $status = Wpfnl_Pro_Webhook_Functions::check_webhook_status( $setting['status'] );
                    if( !$status ){
                        continue;
                    }
                    $this->lms_orderbump_accepted( $cookie_data, $setting , $is_ob_settings );
                    $this->lms_orderbump_accepted_already_enrolled( $cookie_data, $setting , $is_ob_settings );
                    $this->lms_orderbump_rejected( $cookie_data, $setting , $is_ob_settings );
                }
               
               
            }
        }
    }


    /**
     * lms orderbump accepeted
     */
    private function lms_orderbump_accepted( $cookie_data, $setting , $is_ob_settings ){
        $index = str_replace('_order_bump_accepted','',$setting['conditions']);
        $product_id = '';
        $is_ob_key_matched = false;
       
        if( 'any' == $index ){
            if( !empty($cookie_data['ob_accepetd_products']) ){
                $product_id = $cookie_data['ob_accepetd_products'][0];
                $is_ob_key_matched = true;
            }
        }else{
            if( isset($is_ob_settings[$index]) && (isset( $cookie_data['already_enrolled'] ) && !in_array('checkout',$cookie_data['already_enrolled'])) ){
                $product_id = $is_ob_settings[$index]['product'];
                
                if( isset($cookie_data['ob_accepetd_products']) ){
                    if( in_array($product_id,$cookie_data['ob_accepetd_products']) ){
                        $is_ob_key_matched = true;
                    }
                }
                
            }
            
        }
       
        if( $is_ob_key_matched ){
            $is_matched = Wpfnl_Pro_Webhook_Functions::match_conditions( $index.'_order_bump_accepted', $setting['conditions'] );
            if( $is_matched ){
              
                $class = Wpfnl_Pro_Webhook_Functions::get_class_instance( 'orderbump', $setting, [], '' , '', 'accepted', $product_id ,'lms' , $cookie_data );
                if( $class ){
                    $class->send_data(); 
                }
            }   
        }
    }


    /**
     * lms orderbump accepeted
     */
    private function lms_orderbump_accepted_already_enrolled( $cookie_data, $setting , $is_ob_settings ){

        $index = str_replace('_order_bump_enrolled_accepted','',$setting['conditions']);
        $product_id = '';
        $is_ob_key_matched = false;
       
        if( isset($is_ob_settings[$index]) && isset( $cookie_data['already_enrolled'] ) && in_array('checkout',$cookie_data['already_enrolled']) ){
            $product_id = $is_ob_settings[$index]['product'];
            if( in_array($product_id,$cookie_data['ob_accepetd_products']) ){
                $is_ob_key_matched = true;
            }
        }
           
        if( $is_ob_key_matched ){
            $is_matched = Wpfnl_Pro_Webhook_Functions::match_conditions( $index.'_order_bump_enrolled_accepted', $setting['conditions'] );
            if( $is_matched ){
                $class = Wpfnl_Pro_Webhook_Functions::get_class_instance( 'orderbump', $setting, [], '' , '', 'accepted', $product_id ,'lms' , $cookie_data );
                if( $class ){
                    
                    $class->send_data(); 
                }
            }   
        }
    }

    

    /**
     * private lms orderbump accepeted
     */
    private function lms_orderbump_rejected( $cookie_data, $setting , $is_ob_settings ){
        $index = str_replace('_order_bump_rejected','',$setting['conditions']);
        $product_id = '';
        $is_ob_key_matched = false;
        if( 'any' == $index ){
            $is_ob_key_matched = true;
            
        }else{
            if( isset($is_ob_settings[$index])){
                $product_id = $is_ob_settings[$index]['product'];
                if( !in_array($product_id,$cookie_data['ob_accepetd_products']) ){
                    $is_ob_key_matched = true;
                }
            }
            
        }
      
        if( $is_ob_key_matched ){
            $is_matched = Wpfnl_Pro_Webhook_Functions::match_conditions( $index.'_order_bump_rejected', $setting['conditions'] );
            if( $is_matched ){
                
                $class = Wpfnl_Pro_Webhook_Functions::get_class_instance( 'orderbump', $setting, [], '' , '', 'not accepted', $product_id ,'lms' , $cookie_data );
                if( $class ){
                  
                    $class->send_data(); 
                }
            }   
        }
    }


    /**
     * Allow to send data when orderbump triggered
     */
    private function offer_triggered( $cookie_data , $step_id, $step_type ){

        if( $step_id && isset($cookie_data['funnel_id']) ){
            $settings   =  Wpfnl_Pro_Webhook_Functions::get_webhook_settings( $cookie_data['funnel_id'] );
            if( !empty($settings) ){
                foreach( $settings as $key=>$setting ){
                    $status = Wpfnl_Pro_Webhook_Functions::check_webhook_status( $setting['status'] );
                    if( !$status ){
                        continue;
                    }
                    $this->lms_offer_accepted( $cookie_data, $setting , $step_type, $step_id );
                    $this->lms_offer_accepted_already_enrolled( $cookie_data, $setting , $step_type, $step_id);
                    $this->lms_offer_rejected( $cookie_data, $setting , $step_type, $step_id );
                }
               
               
            }
        }
    }


    /**
     * Lms Offer accepted
     */
    private function lms_offer_accepted( $cookie_data, $setting , $step_type, $step_id ){
        
        $is_ob_key_matched = false;
        
        if( isset( $cookie_data['offer'][$step_id] ) && ( isset($cookie_data['already_enrolled']) && !in_array( $step_type,$cookie_data['already_enrolled'])) ){
            $offer_status = 'accepted';
            $is_matched = Wpfnl_Pro_Webhook_Functions::match_conditions( $step_type.'_'.$offer_status, $setting['conditions'] );
          
            if( $is_matched ){
                $is_ob_key_matched = true; 
            }
        }
        $this->create_offer_class_instance( $is_ob_key_matched, $step_type, $setting, $cookie_data ,'accepted' , $step_id);
    }


    /**
     * Lms Offer accepted
     */
    private function lms_offer_accepted_already_enrolled( $cookie_data, $setting , $step_type, $step_id){
        $is_ob_key_matched = false;

        if( isset( $cookie_data['already_enrolled'] ) && in_array( $step_type,$cookie_data['already_enrolled']) ){
            $offer_status = 'accepted';            
            $is_matched = Wpfnl_Pro_Webhook_Functions::match_conditions( $step_type.'_enrolled_'.$offer_status, $setting['conditions'] );
            if( $is_matched ){
                $is_ob_key_matched = true;
            }
        }else{
            $offer_status = 'accepted';            
            $is_matched = Wpfnl_Pro_Webhook_Functions::match_conditions( $step_type.'_enrolled_'.$offer_status, $setting['conditions'] );
            if( $is_matched ){
                $is_ob_key_matched = true;
            }
        }
        $this->create_offer_class_instance( $is_ob_key_matched, $step_type, $setting, $cookie_data, 'accepted', $step_id );
    }


    /**
     * Lms Offer accepted
     */
    private function lms_offer_rejected( $cookie_data, $setting , $step_type, $step_id ){
        $is_ob_key_matched = false;
        if( !isset( $cookie_data['offer'][$step_id] ) ){
            $offer_status = 'rejected';
            $is_matched = Wpfnl_Pro_Webhook_Functions::match_conditions( $step_type.'_'.$offer_status, $setting['conditions'] );
            if( $is_matched ){
                $is_ob_key_matched = true;
            }
        }
        $this->create_offer_class_instance( $is_ob_key_matched, $step_type, $setting, $cookie_data, 'rejected', $step_id );
    }


    /**
     * Create offer class instance
     */
    private function create_offer_class_instance( $is_ob_key_matched, $step_type, $setting, $cookie_data, $status, $step_id){
        if( $is_ob_key_matched ){
            $offer_product = Wpfnl_lms_learndash_functions::get_course_details($step_id);
            $product_id = '';
            if( is_array($offer_product) ){
                if( isset($offer_product['id']) ){
                    $product_id = $offer_product['id'];
                }
            }

            if( $product_id ){
                $class = Wpfnl_Pro_Webhook_Functions::get_class_instance( 'offer', $setting, [], '' , '', $status, $product_id ,'lms' , $cookie_data , $step_id);
                if( $class ){
                    $class->send_data(); 
                }
            }
            
             
        }
    }


}