<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://getwpfunnels.com/
 * @since             1.0.0
 * @package           Wpfunnels_Pro_Lms
 *
 * @wordpress-plugin
 * Plugin Name:       WPFunnels Pro - LMS Funnel
 * Plugin URI:        https://getwpfunnels.com/
 * Description:       Visually plan and craft sales funnels💰 for your online courses 📚 using order bumps, post-purchase upsell & downsell offers, and special discounts.
 * Version:           1.1.2
 * Author:            WPFunnels Team
 * Author URI:        https://getwpfunnels.com/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       wpfunnels-pro-lms
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */

define('WPFNL_PRO_LMS_VERSION', '1.1.2');
define('WPFNL_PRO_LMS_PRODUCT_ID', 'wpf-ld');
$protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";
define('WPFNL_PRO_LMS_INSTANCE', str_replace($protocol, "", get_bloginfo('wpurl')));
define('WPFNL_PRO_LMS_FILE', __FILE__);
define('WPFNL_PRO_LMS_DIR', plugin_dir_path(WPFNL_PRO_LMS_FILE));

//license middleman api url
define('WPFNL_PRO_LMS_LICENSE_URL', 'https://license.getwpfunnels.com/api/v1/licence');

//the url where the WooCommerce Software License plugin is being installed
define('WPFNL_PRO_LMS_API_URL', 'https://useraccount.getwpfunnels.com/');


/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-wpfunnels-pro-lms-activator.php
 */
function activate_wpfunnels_pro_lms() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/utils/class-wpfunnels-pro-lms-activator.php';
	Wpfunnels_Pro_Lms_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-wpfunnels-pro-lms-deactivator.php
 */
function deactivate_wpfunnels_pro_lms() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/utils/class-wpfunnels-pro-lms-deactivator.php';
	Wpfunnels_Pro_Lms_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_wpfunnels_pro_lms' );
register_deactivation_hook( __FILE__, 'deactivate_wpfunnels_pro_lms' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-wpfunnels-pro-lms.php';

function wpfnl_pro_lms_run_updater() {
    new \WPFunnelsPro\Lms\Wpfnl_Pro_Lms_Updater(WPFNL_PRO_LMS_API_URL, 'wpfunnels-pro-lms', 'wpfunnels-pro-lms/wpfunnels-pro-lms.php');
}
add_action('after_setup_theme', 'wpfnl_pro_lms_run_updater');

/**
 * Outputs an admin notice if plugin is trying to be activated when dependent plugin is not activated.
 */
function admin_notice() {
    
    if ( ! function_exists( 'get_plugins' ) || ! function_exists( 'is_plugin_active' ) ) {
        require_once ABSPATH . 'wp-admin/includes/plugin.php';
    }
    $installed_plugins = get_plugins();
    
    if( !isset($installed_plugins['sfwd-lms/sfwd_lms.php']) || !is_plugin_active( 'sfwd-lms/sfwd_lms.php' ) ){
        
        printf( '<div class="error notice is-dismissible"><p class="extension-message">%s</p></div>', 'You must have Learndash to install WPFunnels Pro LMS' );
    }
    
}
add_filter('admin_notices', 'admin_notice');



/**
 * Redirect to license page after plugin activation
 * @since 1.0.9
 */
function wpfnl_pro_lms_redirect() {
    if( is_plugin_active( 'wpfunnels-pro-lms/wpfunnels-pro-lms.php' ) ){
        if (get_option('wpfunnels_pro_lms_do_activation_redirect', false)) {
            delete_option('wpfunnels_pro_lms_do_activation_redirect');

            // On these pages, or during these events, postpone the redirect.
            $do_redirect = true;
            if ( wp_doing_ajax() || is_network_admin() ) {
                $do_redirect = false;
            }
            if( $do_redirect ) {
                wp_redirect("admin.php?page=wpf-license");
            }
        }
    }else{
        delete_option('wpfunnels_pro_lms_do_activation_redirect');
    }
}
add_action('admin_init', 'wpfnl_pro_lms_redirect');


/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_wpfunnels_pro_lms() {

	

    if ( ! function_exists( 'get_plugins' ) || ! function_exists( 'is_plugin_active' ) ) {
        require_once ABSPATH . 'wp-admin/includes/plugin.php';
    }
    $plugin = new Wpfunnels_Pro_Lms();
	$plugin->run();
	
    $dependecy = new \Wpfnl_lms_Activation( 'wpfunnels-pro/wpfnl-pro.php', __FILE__, '1.6.4', 'wpfunnels-pro-lms' );
    $installed_plugins = get_plugins();
	if( !isset($installed_plugins['wpfunnels/wpfnl.php']) || !is_plugin_active( 'wpfunnels/wpfnl.php' ) ){
        $dependecy::deactivate_self( 'wpfunnels-pro-lms/wpfunnels-pro-lms.php' );
    }

    if( !isset($installed_plugins['sfwd-lms/sfwd_lms.php']) || !is_plugin_active( 'sfwd-lms/sfwd_lms.php' ) ){
        $dependecy::deactivate_self( 'wpfunnels-pro-lms/wpfunnels-pro-lms.php' );
    }

    if( !isset($installed_plugins['wpfunnels-pro/wpfnl-pro.php']) || !is_plugin_active( 'wpfunnels-pro/wpfnl-pro.php' ) ){
        $dependecy::deactivate_self( 'wpfunnels-pro-lms/wpfunnels-pro-lms.php' );
    }


    $license   = new \WPFunnelsPro\Lms\Wpfnl_Pro_Lms_Licensing();

}
run_wpfunnels_pro_lms();
